@@ -1,4 +1,4 @@
-/* $Id: blast_returns.c,v 1.12 2004/10/06 15:01:01 dondosha Exp $
+/* $Id: blast_returns.c,v 1.19 2005/04/27 20:00:15 dondosha Exp $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -21,28 +21,25 @@
 *
 *  Please cite the author in any work or product based on this material.
 *
+*  Author: Ilya Dondoshansky
 * ===========================================================================*/
 
-/*****************************************************************************
+/** @file blast_returns.c
+ * Manipulating data returned from BLAST other than Seq-aligns
+ */
 
-File name: blast_returns.c
-
-Author: Ilya Dondoshansky
-
-Contents: Manipulating data returned from BLAST other than Seq-aligns
-
-Detailed Contents: 
-
-******************************************************************************
- * $Revision: 1.12 $
- * */
-
-static char const rcsid[] = "$Id: blast_returns.c,v 1.12 2004/10/06 15:01:01 dondosha Exp $";
+static char const rcsid[] = "$Id: blast_returns.c,v 1.19 2005/04/27 20:00:15 dondosha Exp $";
 
 #include <algo/blast/api/blast_returns.h>
 #include <algo/blast/api/blast_seq.h>
 #include <algo/blast/core/blast_filter.h>
 #include <algo/blast/core/blast_util.h>
+#include <algo/blast/core/pattern.h>
+
+/** @addtogroup CToolkitAlgoBlast
+ *
+ * @{
+ */
 
 TxDfDbInfo* Blast_GetDbInfo(ReadDBFILE* rdfp)
 {
@@ -98,7 +95,8 @@ Blast_GetParametersBuffer(EBlastProgramType program_number,
       raw_cutoffs = diagnostics->cutoffs;
    }
 
-   if (program_number == eBlastTypeBlastn)
+   if (program_number == eBlastTypeBlastn ||
+       program_number == eBlastTypePhiBlastn)
    {
       sprintf(buffer, "Matrix: blastn matrix:%d %d", search_params->match, search_params->mismatch);
       add_string_to_buffer(buffer, &ret_buffer, &ret_buffer_length);
@@ -123,12 +121,16 @@ Blast_GetParametersBuffer(EBlastProgramType program_number,
               Nlm_Int8tostr(ungapped_stats->lookup_hits, 1));
       add_string_to_buffer(buffer, &ret_buffer, &ret_buffer_length);
    
-      sprintf(buffer, "Number of extensions: %ld", 
-              (long) ungapped_stats->init_extends);
-      add_string_to_buffer(buffer, &ret_buffer, &ret_buffer_length);
-      sprintf(buffer, "Number of successful extensions: %ld", 
-              (long) ungapped_stats->good_init_extends);
-      add_string_to_buffer(buffer, &ret_buffer, &ret_buffer_length);
+      if (ungapped_stats->init_extends) {
+          sprintf(buffer, "Number of extensions: %ld", 
+                  (long) ungapped_stats->init_extends);
+          add_string_to_buffer(buffer, &ret_buffer, &ret_buffer_length);
+      }
+      if (ungapped_stats->good_init_extends) {
+          sprintf(buffer, "Number of successful extensions: %ld", 
+                  (long) ungapped_stats->good_init_extends);
+          add_string_to_buffer(buffer, &ret_buffer, &ret_buffer_length);
+      }
    }
 
    if (gapped_stats) {
@@ -144,24 +146,25 @@ Blast_GetParametersBuffer(EBlastProgramType program_number,
       add_string_to_buffer(buffer, &ret_buffer, &ret_buffer_length);
    
       if (search_params->gapped_search) {
-         sprintf(buffer, 
-                 "Number of HSP's better than %4.1f without gapping: %ld", 
-                 search_params->expect, 
-                 (long) gapped_stats->seqs_ungapped_passed);
-         add_string_to_buffer(buffer, &ret_buffer, &ret_buffer_length);
-         sprintf(buffer, 
-                 "Number of HSP's gapped: %ld", 
-                 (long) gapped_stats->extensions);
-         add_string_to_buffer(buffer, &ret_buffer, &ret_buffer_length);
-         sprintf(buffer, 
-                 "Number of HSP's successfully gapped: %ld", 
-                 (long) gapped_stats->good_extensions);
-         add_string_to_buffer(buffer, &ret_buffer, &ret_buffer_length);
-         sprintf(buffer, 
-               "Number of extra gapped extensions for HSPs above %4.1f: %ld", 
-                 search_params->expect,
-                 (long) gapped_stats->extra_extensions);
-         add_string_to_buffer(buffer, &ret_buffer, &ret_buffer_length);
+          if (gapped_stats->seqs_ungapped_passed) {
+              sprintf(buffer, 
+                      "Number of HSP's better than %4.1f without gapping: %ld", 
+                      search_params->expect, 
+                      (long) gapped_stats->seqs_ungapped_passed);
+              add_string_to_buffer(buffer, &ret_buffer, &ret_buffer_length);
+          }
+          sprintf(buffer, "Number of HSP's gapped: %ld", 
+                  (long) gapped_stats->extensions);
+          add_string_to_buffer(buffer, &ret_buffer, &ret_buffer_length);
+          sprintf(buffer, "Number of HSP's successfully gapped: %ld", 
+                  (long) gapped_stats->good_extensions);
+          add_string_to_buffer(buffer, &ret_buffer, &ret_buffer_length);
+          if (gapped_stats->extra_extensions) {
+              sprintf(buffer, 
+                  "Number of extra gapped extensions for HSPs above %4.1f: %ld", 
+                  search_params->expect, (long) gapped_stats->extra_extensions);
+              add_string_to_buffer(buffer, &ret_buffer, &ret_buffer_length);
+          }
       }
    }
 
@@ -185,48 +188,64 @@ Blast_GetParametersBuffer(EBlastProgramType program_number,
       sprintf(buffer, "Effective length of database: %s", 
               Nlm_Int8tostr (db_stats->eff_dblength , 1));
       add_string_to_buffer(buffer, &ret_buffer, &ret_buffer_length);
-      sprintf(buffer, "Effective search space: %8.0f", (double) db_stats->eff_searchsp);
+      sprintf(buffer, "Effective search space: %8.0f", 
+              (double) db_stats->eff_searchsp);
       add_string_to_buffer(buffer, &ret_buffer, &ret_buffer_length);
       sprintf(buffer, "Effective search space used: %8.0f", (double) db_stats->eff_searchsp_used);
       add_string_to_buffer(buffer, &ret_buffer, &ret_buffer_length);
    }
-   sprintf(buffer, "Neighboring words threshold: %ld", 
-           (long) search_params->threshold);
-   add_string_to_buffer(buffer, &ret_buffer, &ret_buffer_length);
-   sprintf(buffer, "Window for multiple hits: %ld", 
-           (long) search_params->window_size);
-   add_string_to_buffer(buffer, &ret_buffer, &ret_buffer_length);
-   
+   if (search_params->threshold) {
+       sprintf(buffer, "Neighboring words threshold: %ld", 
+               (long) search_params->threshold);
+       add_string_to_buffer(buffer, &ret_buffer, &ret_buffer_length);
+   }
+   if (search_params->window_size) {
+       sprintf(buffer, "Window for multiple hits: %ld", 
+               (long) search_params->window_size);
+       add_string_to_buffer(buffer, &ret_buffer, &ret_buffer_length);
+   }
    if (raw_cutoffs) {
       BLAST_KAParameters* ka_params_gap = sum_returns->ka_params_gap;
       BLAST_KAParameters* ka_params = sum_returns->ka_params;
 
-      sprintf(buffer, "X1: %ld (%4.1f bits)", 
-              (long)raw_cutoffs->x_drop_ungapped, (raw_cutoffs->x_drop_ungapped)*(ka_params->Lambda)/NCBIMATH_LN2);
-      add_string_to_buffer(buffer, &ret_buffer, &ret_buffer_length);
-
+      if (raw_cutoffs->x_drop_ungapped) {
+          sprintf(buffer, "X1: %ld (%4.1f bits)", 
+                  (long)raw_cutoffs->x_drop_ungapped, 
+                  (raw_cutoffs->x_drop_ungapped)*(ka_params->Lambda)/NCBIMATH_LN2);
+          add_string_to_buffer(buffer, &ret_buffer, &ret_buffer_length);
+      }
       if (search_params->gapped_search) {
          sprintf(buffer, "X2: %ld (%4.1f bits)", 
                  (long)raw_cutoffs->x_drop_gap, raw_cutoffs->x_drop_gap*(ka_params_gap->Lambda)/NCBIMATH_LN2);
          add_string_to_buffer(buffer, &ret_buffer, &ret_buffer_length);
          sprintf(buffer, "X3: %ld (%4.1f bits)", 
-                 (long)raw_cutoffs->x_drop_gap_final, raw_cutoffs->x_drop_gap_final*(ka_params_gap->Lambda)/NCBIMATH_LN2);
+                 (long)raw_cutoffs->x_drop_gap_final, 
+                 raw_cutoffs->x_drop_gap_final*(ka_params_gap->Lambda) /
+                 NCBIMATH_LN2);
          add_string_to_buffer(buffer, &ret_buffer, &ret_buffer_length);
       
-         sprintf(buffer, "S1: %ld (%4.1f bits)", 
-                 (long)raw_cutoffs->gap_trigger, 
-                 ((raw_cutoffs->gap_trigger*(ka_params->Lambda))-(log(ka_params->K)))/NCBIMATH_LN2);
-         add_string_to_buffer(buffer, &ret_buffer, &ret_buffer_length);
+         if (raw_cutoffs->ungapped_cutoff) {
+             sprintf(buffer, "S1: %ld (%4.1f bits)", 
+                     (long)raw_cutoffs->ungapped_cutoff, 
+                     ((raw_cutoffs->ungapped_cutoff*(ka_params->Lambda)) -
+                      (log(ka_params->K)))/NCBIMATH_LN2);
+             add_string_to_buffer(buffer, &ret_buffer, &ret_buffer_length);
+         }
+      }
+      if (raw_cutoffs->cutoff_score) {
+          if (search_params->gapped_search) {
+              sprintf(buffer, "S2: %ld (%4.1f bits)", 
+                      (long) raw_cutoffs->cutoff_score, 
+                      (((raw_cutoffs->cutoff_score)*(ka_params_gap->Lambda)) -
+                       (log(ka_params_gap->K)))/NCBIMATH_LN2);
+          } else {
+              sprintf(buffer, "S2: %ld (%4.1f bits)", 
+                      (long) raw_cutoffs->cutoff_score, 
+                      (((raw_cutoffs->cutoff_score)*(ka_params->Lambda)) - 
+                       (log(ka_params->K)))/NCBIMATH_LN2);
+          }
+          add_string_to_buffer(buffer, &ret_buffer, &ret_buffer_length);
       }
-
-      if (search_params->gapped_search)
-          sprintf(buffer, "S2: %ld (%4.1f bits)", (long) raw_cutoffs->cutoff_score, 
-            (((raw_cutoffs->cutoff_score)*(ka_params_gap->Lambda))-(log(ka_params_gap->K)))/NCBIMATH_LN2);
-      else
-          sprintf(buffer, "S2: %ld (%4.1f bits)", (long) raw_cutoffs->cutoff_score, 
-            (((raw_cutoffs->cutoff_score)*(ka_params->Lambda))-(log(ka_params->K)))/NCBIMATH_LN2);
-      
-      add_string_to_buffer(buffer, &ret_buffer, &ret_buffer_length);
    }
 
    return ret_buffer;
@@ -239,8 +258,8 @@ Blast_GetParametersBuffer(EBlastProgramType program_number,
  * @param sum_returns Returns summary structure [out]
 */
 static void 
-Blast_SummaryKAParametersFill(const BlastScoreBlk* sbp, Int4 context, 
-                              Blast_SummaryReturn* sum_returns)
+s_SummaryKAParametersFill(const BlastScoreBlk* sbp, Int4 context, 
+                          Blast_SummaryReturn* sum_returns)
 {
    Blast_KarlinBlk* kbp;
 
@@ -250,7 +269,7 @@ Blast_SummaryKAParametersFill(const BlastScoreBlk* sbp, Int4 context,
    if (sbp->kbp) {
       kbp = sbp->kbp[context];
       sum_returns->ka_params = 
-         (BLAST_KAParameters*) malloc(sizeof(BLAST_KAParameters));
+         (BLAST_KAParameters*) calloc(1, sizeof(BLAST_KAParameters));
       sum_returns->ka_params->Lambda = kbp->Lambda;
       sum_returns->ka_params->K = kbp->K;
       sum_returns->ka_params->H = kbp->H;
@@ -263,6 +282,7 @@ Blast_SummaryKAParametersFill(const BlastScoreBlk* sbp, Int4 context,
       sum_returns->ka_params_gap->Lambda = kbp->Lambda;
       sum_returns->ka_params_gap->K = kbp->K;
       sum_returns->ka_params_gap->H = kbp->H;
+      sum_returns->ka_params_gap->C = kbp->paramC; /* Needed only in PHI BLAST */
    }
 }
 
@@ -271,32 +291,30 @@ Blast_SummaryKAParametersFill(const BlastScoreBlk* sbp, Int4 context,
  * @return NULL pointer 
  */
 static Blast_DatabaseStats*
-Blast_SummaryDBStatsFree(Blast_DatabaseStats* db_stats)
+s_SummaryDBStatsFree(Blast_DatabaseStats* db_stats)
 {
     sfree(db_stats);
     return NULL;
 }
 
-/** Allocated and fills in Blast_DatabaseStats
- * Either rdfp or subject_slp should be non-NULL, not both!
- *
+/** Allocates and fills in Blast_DatabaseStats
  * @param program_number blastn/blastp/etc. [in]
  * @param eff_len_options pointer to effective length options [in]
  * @param query_info information about query [in]
- * @param rdfp pointer to BLAST DB reader [in]
- * @param subject_slp Seq-loc for target sequence [in]
+ * @param seq_src Source of subject sequences [in]
  * @param db_stats object to be returned [out]
  * @return zero on success
  */
 static Int2 
-Blast_SummaryDBStatsFill(EBlastProgramType program_number, 
+s_SummaryDBStatsFill(EBlastProgramType program_number, 
    const BlastEffectiveLengthsOptions* eff_len_options,
-   const BlastQueryInfo* query_info, ReadDBFILE* rdfp, 
-   SeqLoc* subject_slp, Blast_DatabaseStats** db_stats)
+   const BlastQueryInfo* query_info, const BlastSeqSrc* seq_src,
+   Blast_DatabaseStats** db_stats)
 {
    Int8 total_length=0;  /* total length of database. */
    Int4 num_entries=0;   /* number of entries in database. */
    Int4 num_frames;      /* number of frames allowed. */
+   
 
    ASSERT(db_stats);
 
@@ -306,12 +324,19 @@ Blast_SummaryDBStatsFill(EBlastProgramType program_number,
 
    if (eff_len_options->db_length) {
       total_length = eff_len_options->db_length;
-   } else if (rdfp) {
-      readdb_get_totals_ex(rdfp, &total_length, &num_entries, TRUE);
    } else {
-      SeqLoc* slp;
-      for (slp = subject_slp; slp; slp = slp->next) {
-         total_length += SeqLocLen(slp);
+      total_length = BlastSeqSrcGetTotLen(seq_src);
+
+      if (total_length == 0) {
+         /* If total length cannot be retrieved directly, calculate it by
+            adding individual sequence lengths. */
+         BlastSeqSrcIterator* itr = BlastSeqSrcIteratorNew();
+         Int4 oid;
+         while ( (oid = BlastSeqSrcIteratorNext(seq_src, itr)) 
+                 != BLAST_SEQSRC_EOF) {
+            total_length += BlastSeqSrcGetSeqLen(seq_src, (void*) &oid);
+         }
+         itr = BlastSeqSrcIteratorFree(itr);
       }
    }
       
@@ -325,28 +350,32 @@ Blast_SummaryDBStatsFill(EBlastProgramType program_number,
 
    if (eff_len_options->dbseq_num)
       num_entries = eff_len_options->dbseq_num;
-   else if (!rdfp) 
-      num_entries = ValNodeLen(subject_slp);
+   else
+      num_entries = BlastSeqSrcGetNumSeqs(seq_src);
 
    (*db_stats)->dbnum = num_entries;
 
    if (program_number == eBlastTypeBlastx ||
        program_number == eBlastTypeTblastx)
       num_frames = NUM_FRAMES;
-   else if (program_number == eBlastTypeBlastn)
+   else if (program_number == eBlastTypeBlastn ||
+            program_number == eBlastTypePhiBlastn)
       num_frames = 2;
    else
       num_frames = 1;
 
    
    if (query_info->last_context < num_frames) {  /* Only one query here. */
-      Int4 qlen = BLAST_GetQueryLength(query_info, query_info->first_context);
-      (*db_stats)->hsp_length = query_info->length_adjustments[query_info->first_context];
+      Int4 qlen = query_info->contexts[query_info->first_context].query_length;
+      (*db_stats)->hsp_length = 
+          query_info->contexts[query_info->first_context].length_adjustment;
       /** FIXME: Should this be different for RPS BLAST? */
       (*db_stats)->qlen = qlen;
       (*db_stats)->eff_qlen = qlen - ((*db_stats)->hsp_length);
-      (*db_stats)->eff_dblength = total_length - num_entries*((*db_stats)->hsp_length);
-      (*db_stats)->eff_searchsp = query_info->eff_searchsp_array[query_info->first_context];
+      (*db_stats)->eff_dblength = 
+          total_length - num_entries*((*db_stats)->hsp_length);
+      (*db_stats)->eff_searchsp = 
+          query_info->contexts[query_info->first_context].eff_searchsp;
       if (eff_len_options && eff_len_options->searchsp_eff)
          (*db_stats)->eff_searchsp_used = eff_len_options->searchsp_eff;
       else
@@ -362,7 +391,7 @@ Blast_SummaryDBStatsFill(EBlastProgramType program_number,
  * @return NULL pointer 
  */
 static Blast_SearchParams*
-Blast_SummarySearchParamsFree(Blast_SearchParams* search_params)
+s_SummarySearchParamsFree(Blast_SearchParams* search_params)
 {
 
    if (search_params == NULL)
@@ -371,6 +400,7 @@ Blast_SummarySearchParamsFree(Blast_SearchParams* search_params)
    sfree(search_params->matrix);
    sfree(search_params->entrez_query);
    sfree(search_params->filter_string);
+   sfree(search_params->pattern);
    sfree(search_params);
 
    return NULL;
@@ -389,7 +419,7 @@ Blast_SummarySearchParamsFree(Blast_SearchParams* search_params)
  * @return zero on success
  */
 static Int2 
-Blast_SummarySearchParamsFill(EBlastProgramType program_number,
+s_SummarySearchParamsFill(EBlastProgramType program_number,
    const BlastScoringOptions* score_options,
    const LookupTableOptions* lookup_options,
    const BlastHitSavingOptions* hit_options,
@@ -407,7 +437,8 @@ Blast_SummarySearchParamsFill(EBlastProgramType program_number,
    if (search_params_lcl == NULL)
      return -1;
 
-   if (program_number == eBlastTypeBlastn)
+   if (program_number == eBlastTypeBlastn ||
+       program_number == eBlastTypePhiBlastn)
    {
       search_params_lcl->match = score_options->reward;
       search_params_lcl->mismatch = score_options->penalty;
@@ -446,19 +477,33 @@ Blast_SummarySearchParamsFill(EBlastProgramType program_number,
    return 0;
 }
 
+Blast_SummaryReturn* Blast_SummaryReturnNew()
+{
+    return (Blast_SummaryReturn*) calloc(1, sizeof(Blast_SummaryReturn));
+}
 
-Blast_SummaryReturn* 
-Blast_SummaryReturnFree(Blast_SummaryReturn* sum_returns)
+void 
+Blast_SummaryReturnClean(Blast_SummaryReturn* sum_returns)
 {
    if (sum_returns) {
       sfree(sum_returns->ka_params);
       sfree(sum_returns->ka_params_gap);
-      sum_returns->db_stats = Blast_SummaryDBStatsFree(sum_returns->db_stats);
-      sum_returns->search_params = Blast_SummarySearchParamsFree(sum_returns->search_params);
+      sum_returns->db_stats = s_SummaryDBStatsFree(sum_returns->db_stats);
+      sum_returns->search_params = 
+          s_SummarySearchParamsFree(sum_returns->search_params);
       sum_returns->diagnostics = Blast_DiagnosticsFree(sum_returns->diagnostics);
-      sfree(sum_returns);
+      sum_returns->error = Blast_MessageFree(sum_returns->error);
+      sum_returns->pattern_info = 
+          SPHIQueryInfoFree(sum_returns->pattern_info);
    }
-   return NULL;
+}
+
+Blast_SummaryReturn* 
+Blast_SummaryReturnFree(Blast_SummaryReturn* sum_returns)
+{
+    Blast_SummaryReturnClean(sum_returns);
+    sfree(sum_returns);
+    return NULL;
 }
 
 Int2 Blast_SummaryReturnFill(EBlastProgramType program_number, 
@@ -471,29 +516,23 @@ Int2 Blast_SummaryReturnFill(EBlastProgramType program_number,
         const BlastEffectiveLengthsOptions* eff_len_options,
         const QuerySetUpOptions* query_setup,
         const BlastQueryInfo* query_info,
-        ReadDBFILE* rdfp, SeqLoc* subject_slp,
+        const BlastSeqSrc* seq_src,
         BlastDiagnostics** diagnostics,
-        Blast_SummaryReturn** sum_returns_out)
+        Blast_SummaryReturn* sum_returns)
 {
-   Blast_SummaryReturn* sum_returns = 
-      (Blast_SummaryReturn*) calloc(1, sizeof(Blast_SummaryReturn));
-
-   if (sum_returns_out == NULL)
-      return -1;
+    if (!sum_returns)
+        return -1;
 
    if (score_options == NULL || sbp == NULL || lookup_options == NULL || 
       word_options == NULL || ext_options == NULL || hit_options == NULL ||
       eff_len_options == NULL || query_info == NULL)
      return -1;
 
-   if (rdfp == NULL && subject_slp == NULL)
-     return -1;
-
-   Blast_SummaryKAParametersFill(sbp, query_info->first_context, sum_returns);
-   Blast_SummaryDBStatsFill(program_number, eff_len_options, query_info, rdfp, 
-            subject_slp, &(sum_returns->db_stats));
+   s_SummaryKAParametersFill(sbp, query_info->first_context, sum_returns);
+   s_SummaryDBStatsFill(program_number, eff_len_options, query_info, 
+                            seq_src, &(sum_returns->db_stats));
    
-   Blast_SummarySearchParamsFill(program_number, score_options,
+   s_SummarySearchParamsFill(program_number, score_options,
             lookup_options, hit_options, query_setup, word_options,
             NULL, &(sum_returns->search_params));
    if (diagnostics)
@@ -501,7 +540,16 @@ Int2 Blast_SummaryReturnFill(EBlastProgramType program_number,
       sum_returns->diagnostics = *diagnostics;
       *diagnostics = NULL;
    }
-   *sum_returns_out = sum_returns;
+
+   if (program_number == eBlastTypePhiBlastn || 
+       program_number == eBlastTypePhiBlastp) {
+       /* Copy the pattern information structure. */
+       sum_returns->pattern_info = 
+           SPHIQueryInfoCopy(query_info->pattern_info);
+   }
 
    return 0;
 }
+
+/* @} */
+
