@@ -1,4 +1,4 @@
-/*  $Id: seqsrc_readdb.c,v 1.35 2004/10/06 14:59:28 dondosha Exp $
+/*  $Id: seqsrc_readdb.c,v 1.49 2005/04/21 15:00:36 dondosha Exp $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -21,26 +21,31 @@
 *
 *  Please cite the author in any work or product based on this material.
 *
-* ===========================================================================
-*
-* Author:  Christiam Camacho
-*
-* File Description:
-*   Implementation of the BlastSeqSrc interface using readdb
-*
-*/
+*  Author:  Christiam Camacho
+* ===========================================================================*/
 
-static char const rcsid[] = "$Id: seqsrc_readdb.c,v 1.35 2004/10/06 14:59:28 dondosha Exp $";
+/** @file seqsrc_readdb.c
+ * Implementation of the BlastSeqSrc interface using readdb
+ */
+
+static char const rcsid[] = "$Id: seqsrc_readdb.c,v 1.49 2005/04/21 15:00:36 dondosha Exp $";
 
 #include <algo/blast/api/seqsrc_readdb.h>
+#include <algo/blast/core/blast_seqsrc_impl.h>
 #include <algo/blast/core/blast_def.h>
 #include <algo/blast/core/blast_util.h>
 
+/** @addtogroup CToolkitAlgoBlast
+ *
+ * @{
+ */
+
 /** Retrieves the length of the longest sequence in the BlastSeqSrc.
  * @param readdb_handle Pointer to initialized ReadDBFILEPtr structure [in]
  * @param ignoreme Unused by this implementation [in]
  */
-static Int4 ReaddbGetMaxLength(void* readdb_handle, void* ignoreme)
+static Int4 
+s_ReaddbGetMaxLength(void* readdb_handle, void* ignoreme)
 {
     ReadDBFILEPtr rdfp = (ReadDBFILEPtr) readdb_handle;
     Int4 retval = 0;
@@ -55,7 +60,8 @@ static Int4 ReaddbGetMaxLength(void* readdb_handle, void* ignoreme)
  * @param readdb_handle Pointer to initialized ReadDBFILEPtr structure [in]
  * @param ignoreme Unused by this implementation [in]
  */
-static Int4 ReaddbGetNumSeqs(void* readdb_handle, void* ignoreme)
+static Int4 
+s_ReaddbGetNumSeqs(void* readdb_handle, void* ignoreme)
 {
    ReadDBFILEPtr rdfp = (ReadDBFILEPtr) readdb_handle;
    Int4 dbnseqs = 0;
@@ -69,7 +75,8 @@ static Int4 ReaddbGetNumSeqs(void* readdb_handle, void* ignoreme)
  * @param readdb_handle Pointer to initialized ReadDBFILEPtr structure [in]
  * @param ignoreme Unused by this implementation [in]
  */
-static Int8 ReaddbGetTotLen(void* readdb_handle, void* ignoreme)
+static Int8 
+s_ReaddbGetTotLen(void* readdb_handle, void* ignoreme)
 {
     ReadDBFILEPtr rdfp = (ReadDBFILEPtr) readdb_handle;
     Int4 dbnseqs = 0;
@@ -83,10 +90,11 @@ static Int8 ReaddbGetTotLen(void* readdb_handle, void* ignoreme)
  * @param readdb_handle Pointer to initialized ReadDBFILEPtr structure [in]
  * @param ignoreme Unused by this implementation [in]
  */
-static Int4 ReaddbGetAvgLength(void* readdb_handle, void* ignoreme)
+static Int4 
+s_ReaddbGetAvgLength(void* readdb_handle, void* ignoreme)
 {
-   Int8 total_length = ReaddbGetTotLen(readdb_handle, ignoreme);
-   Int4 num_seqs = MAX(1, ReaddbGetNumSeqs(readdb_handle, ignoreme));
+   Int8 total_length = s_ReaddbGetTotLen(readdb_handle, ignoreme);
+   Int4 num_seqs = MAX(1, s_ReaddbGetNumSeqs(readdb_handle, ignoreme));
 
    return (Int4) (total_length/num_seqs);
 }
@@ -95,7 +103,8 @@ static Int4 ReaddbGetAvgLength(void* readdb_handle, void* ignoreme)
  * @param readdb_handle Pointer to initialized ReadDBFILEPtr structure [in]
  * @param ignoreme Unused by this implementation [in]
  */
-static const char* ReaddbGetName(void* readdb_handle, void* ignoreme)
+static const char* 
+s_ReaddbGetName(void* readdb_handle, void* ignoreme)
 {
     ReadDBFILEPtr rdfp = (ReadDBFILEPtr) readdb_handle;
 
@@ -106,7 +115,8 @@ static const char* ReaddbGetName(void* readdb_handle, void* ignoreme)
  * @param readdb_handle Pointer to initialized ReadDBFILEPtr structure [in]
  * @param ignoreme Unused by this implementation [in]
  */
-static Boolean ReaddbGetIsProt(void* readdb_handle, void* ignoreme)
+static Boolean 
+s_ReaddbGetIsProt(void* readdb_handle, void* ignoreme)
 {
     ReadDBFILEPtr rdfp = (ReadDBFILEPtr) readdb_handle;
 
@@ -115,13 +125,14 @@ static Boolean ReaddbGetIsProt(void* readdb_handle, void* ignoreme)
 
 /** Retrieves the sequence meeting the criteria defined by its second argument.
  * @param readdb_handle Pointer to initialized ReadDBFILEPtr structure [in]
- * @param args Pointer to GetSeqArg structure [in]
+ * @param args Pointer to BlastSeqSrcGetSeqArg structure [in]
  * @return return codes defined in blast_seqsrc.h
  */
-static Int2 ReaddbGetSequence(void* readdb_handle, void* args)
+static Int2 
+s_ReaddbGetSequence(void* readdb_handle, void* args)
 {
     ReadDBFILEPtr rdfp = (ReadDBFILEPtr) readdb_handle;
-    GetSeqArg* readdb_args = (GetSeqArg*) args;
+    BlastSeqSrcGetSeqArg* readdb_args = (BlastSeqSrcGetSeqArg*) args;
     Int4 oid = -1, len = 0, buflen = 0;
     Uint1 *buf = NULL, encoding;
     Boolean has_sentinel_byte;
@@ -168,15 +179,15 @@ static Int2 ReaddbGetSequence(void* readdb_handle, void* args)
 
 /** Deallocates uncompressed sequence buffer, obtained by ReaddbGetSequence.
  * @param readdb_handle Pointer to initialized ReadDBFILEPtr structure [in]
- * @param args Pointer to GetSeqArg structure [in]
- * @return return codes defined in blast_seqsrc.h
+ * @param args Pointer to BlastSeqSrcGetSeqArg structure [in]
  */
-static Int2 ReaddbRetSequence(void* readdb_handle, void* args)
+static void
+s_ReaddbReleaseSequence(void* readdb_handle, void* args)
 {
-    GetSeqArg* readdb_args = (GetSeqArg*) args;
-
+    BlastSeqSrcGetSeqArg* readdb_args = (BlastSeqSrcGetSeqArg*) args;
     ASSERT(readdb_args);
-    return BlastSequenceBlkClean(readdb_args->seq);
+    ASSERT(readdb_args->seq);
+    BlastSequenceBlkClean(readdb_args->seq);
 }
 
 /** Retrieve length of a given database sequence.
@@ -184,7 +195,8 @@ static Int2 ReaddbRetSequence(void* readdb_handle, void* args)
  * @param args Pointer to integer indicating ordinal id [in]
  * @return Length of the database sequence or BLAST_SEQSRC_ERROR.
  */
-static Int4 ReaddbGetSeqLen(void* readdb_handle, void* args)
+static Int4 
+s_ReaddbGetSeqLen(void* readdb_handle, void* args)
 {
     ReadDBFILEPtr rdfp = (ReadDBFILEPtr) readdb_handle;
     Int4* oid = (Int4*) args;
@@ -195,15 +207,6 @@ static Int4 ReaddbGetSeqLen(void* readdb_handle, void* args)
     return readdb_get_sequence_length(rdfp, *oid);
 }
 
-/* There are no error messages saved in the ReadDBFILE structure, so the 
- * following getter function is implemented as always returning NULL.
- * @todo Should more meaningful error reporting be implemented?
- */
-static Blast_Message* ReaddbGetError(void* readdb_handle, void* args)
-{
-   return NULL;
-}
-
 /** Mutex for retrieving ordinal id chunks from ReadDB in a multi-threaded
  * search.
  */
@@ -216,17 +219,18 @@ static TNlmMutex ReaddbMutex;
  * any call to this function.
  * @param rdfp List of ReadDBFILE structures [in]
  * @param itr BLAST sequence source iterator [in]
+ * @param last_oid_assigned Last ordinal id processed [in] [out]
  * @return Status
  */ 
 static Int2
-ReadDbGetNextOidListChunk(ReadDBFILEPtr rdfp, BlastSeqSrcIterator* itr,
+s_ReadDbGetNextOidListChunk(ReadDBFILEPtr rdfp, BlastSeqSrcIterator* itr,
                           Uint4* last_oid_assigned)
      
 {
    Int2 status = BLAST_SEQSRC_SUCCESS;
    OIDListPtr oidlist;
    Uint4  gi_start, gi_end;
-   Uint4* id_list;
+   Int4* id_list;
    Uint4 oidindex  = 0;
 
    if (!itr || !last_oid_assigned)
@@ -287,13 +291,20 @@ ReadDbGetNextOidListChunk(ReadDBFILEPtr rdfp, BlastSeqSrcIterator* itr,
    return status;
 }
 
-
-
-static Int2 ReaddbGetNextChunk(void* readdb_handle, BlastSeqSrcIterator* itr)
+/** Assigns next chunk of ordinal ids from the database to the sequence source
+ * iterator.
+ * @param readdb_handle Pointer to the ReadDBFILE structure [in]
+ * @param itr Iterator over the sequence source associated with the 
+ *            readdb_handle, requesting the next chunk of the 
+ *            database.  [in] [out]
+ * @return Status.
+ */
+static Int2 
+s_ReaddbGetNextChunk(void* readdb_handle, BlastSeqSrcIterator* itr)
 {
     ReadDBFILEPtr rdfp = (ReadDBFILEPtr) readdb_handle;
     ReadDBFILEPtr rdfp_head = rdfp;
-    unsigned int nseqs;
+    unsigned int nseqs = 0;
     Uint4 current_oid;
     Int2 status = BLAST_SEQSRC_SUCCESS;
     Uint4 real_readdb_entries;
@@ -333,7 +344,7 @@ static Int2 ReaddbGetNextChunk(void* readdb_handle, BlastSeqSrcIterator* itr)
        itr->oid_range[1] = MIN(current_oid + itr->chunk_sz, nseqs);
        rdfp_head->shared_info->last_oid_assigned = itr->oid_range[1];
     } else {
-       status = ReadDbGetNextOidListChunk(rdfp, itr, 
+       status = s_ReadDbGetNextOidListChunk(rdfp, itr, 
                    &rdfp_head->shared_info->last_oid_assigned);
     }
 
@@ -342,20 +353,26 @@ static Int2 ReaddbGetNextChunk(void* readdb_handle, BlastSeqSrcIterator* itr)
     return status;
 }
 
-static Int4 ReaddbIteratorNext(void* seqsrc, BlastSeqSrcIterator* itr)
+/** Given an iterator over a BLAST database, returns the next ordinal id 
+ * to search.
+ * @param readdb_handle Pointer to the ReadDBFILE structure [in]
+ * @param itr Iterator over seqsrc. [in]
+ * @return Next ordinal id to search.
+ */
+static Int4 
+s_ReaddbIteratorNext(void* readdb_handle, BlastSeqSrcIterator* itr)
 {
-    BlastSeqSrc* bssp = (BlastSeqSrc*) seqsrc;
     Int4 retval = BLAST_SEQSRC_EOF;
     Int4 status = BLAST_SEQSRC_SUCCESS;
     Uint4 last_pos = 0;
 
-    ASSERT(bssp);
+    ASSERT(readdb_handle);
     ASSERT(itr);
 
     /* If iterator is uninitialized/invalid, retrieve the next chunk from the
      * BlastSeqSrc */
     if (itr->current_pos == UINT4_MAX) {
-        status = BLASTSeqSrcGetNextChunk(bssp, itr);
+        status = s_ReaddbGetNextChunk(readdb_handle, itr);
         if (status != BLAST_SEQSRC_SUCCESS) {
             return status;
         }
@@ -381,36 +398,140 @@ static Int4 ReaddbIteratorNext(void* seqsrc, BlastSeqSrcIterator* itr)
     return retval;
 }
 
-BlastSeqSrc* ReaddbSeqSrcNew(BlastSeqSrc* retval, void* args)
+/** Readdb sequence source destructor: frees its internal data structure and the
+ * BlastSeqSrc structure itself.
+ * @param bssp BlastSeqSrc structure to free [in]
+ * @return NULL
+ */
+static BlastSeqSrc* 
+s_ReaddbSeqSrcFree(BlastSeqSrc* bssp)
+{
+    if (!bssp) 
+        return NULL;
+    readdb_destruct((ReadDBFILEPtr)_BlastSeqSrcImpl_GetDataStructure(bssp));
+    sfree(bssp);
+    return NULL;
+}
+
+/** Readdb sequence source copier: 
+ * creates a new copy of the ReadDBFILE structure by calling readdb_attach.
+ * @param bssp BlastSeqSrc structure to copy [in]
+ * @return New BlastSeqSrc structure
+ */
+static BlastSeqSrc* 
+s_ReaddbSeqSrcCopy(BlastSeqSrc* bssp)
+{
+   ReadDBFILE* rdfp = NULL;
+
+   if (!bssp) 
+      return NULL;
+
+   rdfp = readdb_attach((ReadDBFILEPtr)_BlastSeqSrcImpl_GetDataStructure(bssp));
+
+   _BlastSeqSrcImpl_SetDataStructure(bssp, (void*) rdfp);
+    
+   return bssp;
+}
+
+/** Initializes function pointers and data structure in a new readdb-based 
+ * BlastSeqSrc.
+ * @param retval Allocated BlastSeqSrc structure [in|out]
+ * @param rdfp ReadDBFILE structure to be used as data structure. [in]
+ */
+static void
+s_InitNewReaddbSeqSrc(BlastSeqSrc* retval, ReadDBFILE* rdfp)
+{
+    ASSERT(retval);
+
+    /* Initialize the BlastSeqSrc structure fields with user-defined function
+     * pointers and rdfp */
+    _BlastSeqSrcImpl_SetDeleteFnPtr(retval, &s_ReaddbSeqSrcFree);
+    _BlastSeqSrcImpl_SetCopyFnPtr(retval, &s_ReaddbSeqSrcCopy);
+    _BlastSeqSrcImpl_SetDataStructure(retval, (void*) rdfp);
+    _BlastSeqSrcImpl_SetGetNumSeqs(retval, &s_ReaddbGetNumSeqs);
+    _BlastSeqSrcImpl_SetGetMaxSeqLen(retval, &s_ReaddbGetMaxLength);
+    _BlastSeqSrcImpl_SetGetAvgSeqLen(retval, &s_ReaddbGetAvgLength);
+    _BlastSeqSrcImpl_SetGetTotLen(retval, &s_ReaddbGetTotLen);
+    _BlastSeqSrcImpl_SetGetName(retval, &s_ReaddbGetName);
+    _BlastSeqSrcImpl_SetGetIsProt(retval, &s_ReaddbGetIsProt);
+    _BlastSeqSrcImpl_SetGetSequence(retval, &s_ReaddbGetSequence);
+    _BlastSeqSrcImpl_SetGetSeqLen(retval, &s_ReaddbGetSeqLen);
+    _BlastSeqSrcImpl_SetIterNext(retval, &s_ReaddbIteratorNext);
+    _BlastSeqSrcImpl_SetReleaseSequence(retval, &s_ReaddbReleaseSequence);
+}
+
+/** Fills contents in the allocated BlastSeqSrc, given an already created
+ * ReadDBFILE structure. Uses readdb_attach to ensure that input ReadDBFILE
+ * is not freed in BlastSeqSrcFree.
+ * @param retval Allocated BlastSeqSrc to fill [in|out]
+ * @param args Pointer to a ReadDBFILE structure, cast to void to ensure 
+ *             correct signature. [in]
+ * @return Same as retval.
+ */
+static BlastSeqSrc* 
+s_ReaddbSeqSrcAttach(BlastSeqSrc* retval, void* args)
+{
+    ReadDBFILE* rdfp_in = (ReadDBFILE*) args;
+    ReadDBFILE* rdfp = NULL;
+
+    ASSERT(retval);
+
+    if (!rdfp_in) {
+        _BlastSeqSrcImpl_SetInitErrorStr(retval, 
+         strdup("Cannot attach BlastSeqSrc to a NULL ReadDBFILE structure"));
+    } else {
+        rdfp = readdb_attach(rdfp_in);
+        if ( !rdfp ) {
+            _BlastSeqSrcImpl_SetInitErrorStr(retval, 
+             strdup("Failed to attach to existing ReadDBFILE structure"));
+        }
+    }
+
+    s_InitNewReaddbSeqSrc(retval, rdfp);
+
+    return retval;
+}
+
+/** Encapsulates the arguments needed to initialize a BLAST database using
+ * readdb */
+typedef struct ReaddbNewArgs {
+    char* dbname;     /**< Database name */
+    Boolean is_protein; /**< Is this database protein? */
+   Int4 first_db_seq; /**< Ordinal id of the first sequence to search */
+   Int4 final_db_seq; /**< Ordinal id of the last sequence to search */
+} ReaddbNewArgs;
+
+/** Readdb sequence source constructor 
+ * @param retval BlastSeqSrc structure (already allocated) to populate [in]
+ * @param args Pointer to ReaddbNewArgs structure above [in]
+ * @return Updated bssp structure (with all function pointers initialized
+ */
+static BlastSeqSrc* 
+s_ReaddbSeqSrcNew(BlastSeqSrc* retval, void* args)
 {
     ReaddbNewArgs* rargs = (ReaddbNewArgs*) args;
     ReadDBFILEPtr rdfp = NULL;
 
-    if (!retval)
-        return NULL;
+    ASSERT(retval);
+    ASSERT(rargs);
 
-    /* Initialize the rdfp */
-    if ( !(rdfp = readdb_new(rargs->dbname, rargs->is_protein)))
-        return NULL;
+    if ( !rargs->dbname ) {
+        _BlastSeqSrcImpl_SetInitErrorStr(retval, 
+         strdup("Cannot initialize readdb BlastSeqSrc "
+               "with NULL database name"));
+    } else {
+        /* Initialize the rdfp */
+        if ( !(rdfp = readdb_new(rargs->dbname, rargs->is_protein))) {
+            char buf[1024];
+            snprintf(buf, sizeof(buf), 
+                     "s_ReaddbSeqSrcNew: could not open %s %s database\n",
+                     rargs->dbname, 
+                     (rargs->is_protein ? "protein" : "nucleotide"));
+            _BlastSeqSrcImpl_SetInitErrorStr(retval, strdup(buf));
+        }
+    }
 
-    /* Initialize the BlastSeqSrc structure fields with user-defined function
-     * pointers and rdfp */
-    SetDeleteFnPtr(retval, &ReaddbSeqSrcFree);
-    SetCopyFnPtr(retval, &ReaddbSeqSrcCopy);
-    SetDataStructure(retval, (void*) rdfp);
-    SetGetNumSeqs(retval, &ReaddbGetNumSeqs);
-    SetGetMaxSeqLen(retval, &ReaddbGetMaxLength);
-    SetGetAvgSeqLen(retval, &ReaddbGetAvgLength);
-    SetGetTotLen(retval, &ReaddbGetTotLen);
-    SetGetAvgSeqLen(retval, &ReaddbGetAvgLength);
-    SetGetName(retval, &ReaddbGetName);
-    SetGetIsProt(retval, &ReaddbGetIsProt);
-    SetGetSequence(retval, &ReaddbGetSequence);
-    SetGetSeqLen(retval, &ReaddbGetSeqLen);
-    SetGetNextChunk(retval, &ReaddbGetNextChunk);
-    SetIterNext(retval, &ReaddbIteratorNext);
-    SetGetError(retval, &ReaddbGetError);
-    SetRetSequence(retval, &ReaddbRetSequence);
+    s_InitNewReaddbSeqSrc(retval, rdfp);
 
     /* Set the range, if it is specified */
     if (rargs->first_db_seq > 0) {
@@ -435,46 +556,37 @@ BlastSeqSrc* ReaddbSeqSrcNew(BlastSeqSrc* retval, void* args)
     return retval;
 }
 
-BlastSeqSrc* ReaddbSeqSrcFree(BlastSeqSrc* bssp)
-{
-    if (!bssp) 
-        return NULL;
-    readdb_destruct((ReadDBFILEPtr)GetDataStructure(bssp));
-    sfree(bssp);
-    return NULL;
-}
-
-BlastSeqSrc* ReaddbSeqSrcCopy(BlastSeqSrc* bssp)
-{
-   ReadDBFILE* rdfp = NULL;
-
-   if (!bssp) 
-      return NULL;
-
-   rdfp = readdb_attach((ReadDBFILEPtr)GetDataStructure(bssp));
-
-   SetDataStructure(bssp, (void*) rdfp);
-    
-   return bssp;
-}
-
 BlastSeqSrc* 
 ReaddbBlastSeqSrcInit(const char* dbname, Boolean is_prot, int first_seq, 
-                      int last_seq, void* extra_arg)
+                      int last_seq)
 {
     BlastSeqSrcNewInfo bssn_info;
     BlastSeqSrc* seq_src = NULL;
-    ReaddbNewArgs* readdb_args = 
-        (ReaddbNewArgs*) calloc(1, sizeof(ReaddbNewArgs));;
-    readdb_args->dbname = strdup(dbname);
-    readdb_args->is_protein = is_prot;
-    readdb_args->first_db_seq = first_seq;
-    readdb_args->final_db_seq = last_seq;
-    bssn_info.constructor = &ReaddbSeqSrcNew;
-    bssn_info.ctor_argument = (void*) readdb_args;
+    ReaddbNewArgs readdb_args;
+
+    readdb_args.dbname = (char*) dbname;
+    readdb_args.is_protein = is_prot;
+    readdb_args.first_db_seq = first_seq;
+    readdb_args.final_db_seq = last_seq;
+    bssn_info.constructor = &s_ReaddbSeqSrcNew;
+    bssn_info.ctor_argument = (void*) &readdb_args;
 
     seq_src = BlastSeqSrcNew(&bssn_info);
-    sfree(readdb_args->dbname);
-    sfree(readdb_args);
     return seq_src;
 }
+
+BlastSeqSrc*
+ReaddbBlastSeqSrcAttach(ReadDBFILE* rdfp)
+{
+    BlastSeqSrcNewInfo bssn_info;
+    BlastSeqSrc* seq_src = NULL;
+
+    bssn_info.constructor = &s_ReaddbSeqSrcAttach;
+    bssn_info.ctor_argument = (void*) rdfp;
+
+    seq_src = BlastSeqSrcNew(&bssn_info);
+    return seq_src;
+}
+
+/* @} */
+
