@@ -1,4 +1,4 @@
-/* $Id: blast_engine.h,v 1.44 2004/09/07 17:20:01 dondosha Exp $
+/* $Id: blast_engine.h,v 1.49 2005/03/28 21:22:50 dondosha Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -28,7 +28,7 @@
  */
 
 /** @file blast_engine.h
- * High level BLAST functions
+* Function calls to actually perform a BLAST search (high level).
  */
 
 #ifndef __BLAST_ENGINE__
@@ -62,12 +62,13 @@ extern "C" {
  * @param psi_options Options specific to PSI-BLAST [in]
  * @param db_options Options for handling BLAST database [in]
  * @param hsp_stream Structure for streaming results [in] [out]
+ * @param rps_info RPS BLAST auxiliary data structure [in]
  * @param diagnostics Return statistics containing numbers of hits on 
  *                    different stages of the search [out]
  * @param results Results of the BLAST search [out]
  */
 Int4 
-BLAST_SearchEngine(EBlastProgramType program_number, 
+Blast_RunFullSearch(EBlastProgramType program_number, 
    BLAST_SequenceBlk* query, BlastQueryInfo* query_info,
    const BlastSeqSrc* seq_src, BlastScoreBlk* sbp, 
    const BlastScoringOptions* score_options, 
@@ -78,8 +79,8 @@ BLAST_SearchEngine(EBlastProgramType program_number,
    const BlastEffectiveLengthsOptions* eff_len_options,
    const PSIBlastOptions* psi_options, 
    const BlastDatabaseOptions* db_options,
-   BlastHSPStream* hsp_stream, BlastDiagnostics* diagnostics,
-   BlastHSPResults** results);
+   BlastHSPStream* hsp_stream, const BlastRPSInfo* rps_info,
+   BlastDiagnostics* diagnostics, BlastHSPResults** results);
 
 /** The high level function performing an RPS BLAST search 
  * @param program_number Type of BLAST program [in]
@@ -94,7 +95,6 @@ BLAST_SearchEngine(EBlastProgramType program_number,
  * @param hit_options Options for saving the HSPs [in]
  * @param eff_len_options Options for setting effective lengths [in]
  * @param psi_options Options specific to PSI-BLAST [in]
- * @param db_options Options for handling BLAST database [in]
  * @param hsp_stream Placeholder for saving results [in]
  * @param diagnostics Return statistics containing numbers of hits on 
  *                    different stages of the search [out]
@@ -111,7 +111,6 @@ BLAST_RPSSearchEngine(EBlastProgramType program_number,
    const BlastHitSavingOptions* hit_options,
    const BlastEffectiveLengthsOptions* eff_len_options,
    const PSIBlastOptions* psi_options, 
-   const BlastDatabaseOptions* db_options,
    BlastHSPStream* hsp_stream, BlastDiagnostics* diagnostics, 
    BlastHSPResults** results);
 
@@ -149,18 +148,63 @@ BLAST_PreliminarySearchEngine(EBlastProgramType program_number,
    const BlastDatabaseOptions* db_options,
    BlastHSPStream* hsp_stream, BlastDiagnostics* diagnostics);
 
+/** The high level function performing the BLAST search against a BLAST 
+ * database after all the setup has been done.
+ * @param program_number Type of BLAST program [in]
+ * @param query The query sequence [in]
+ * @param query_info Additional query information [in]
+ * @param seq_src Structure containing BLAST database [in]
+ * @param score_options Hit scoring options [in]
+ * @param sbp Scoring and statistical parameters [in]
+ * @param lookup_wrap The lookup table, constructed earlier [in] 
+ * @param word_options Options for processing initial word hits [in]
+ * @param ext_options Options and parameters for the gapped extension [in]
+ * @param hit_options Options for saving the HSPs [in]
+ * @param eff_len_options Options for setting effective lengths [in]
+ * @param psi_options Options specific to PSI-BLAST [in]
+ * @param db_options Options for handling BLAST database [in]
+ * @param hsp_stream Structure for streaming results [in] [out]
+ * @param diagnostics Return statistics containing numbers of hits on 
+ *                    different stages of the search [out]
+ */
+Int2 
+Blast_RunPreliminarySearch(EBlastProgramType program, 
+   BLAST_SequenceBlk* query, BlastQueryInfo* query_info, 
+   const BlastSeqSrc* seq_src, const BlastScoringOptions* score_options,
+   BlastScoreBlk* sbp, LookupTableWrap* lookup_wrap,
+   const BlastInitialWordOptions* word_options, 
+   const BlastExtensionOptions* ext_options,
+   const BlastHitSavingOptions* hit_options,
+   const BlastEffectiveLengthsOptions* eff_len_options,
+   const PSIBlastOptions* psi_options, const BlastDatabaseOptions* db_options, 
+   BlastHSPStream* hsp_stream, BlastDiagnostics* diagnostics);
+
+
 /** Gapped extension function pointer type */
 typedef Int2 (*BlastGetGappedScoreType) 
-     (EBlastProgramType, BLAST_SequenceBlk*, BlastQueryInfo* query_info,
-      BLAST_SequenceBlk*, BlastGapAlignStruct*, const BlastScoringParameters*,
-      const BlastExtensionParameters*, const BlastHitSavingParameters*,
-      BlastInitHitList*, BlastHSPList**, BlastGappedStats*);
+     (EBlastProgramType, /**< @todo comment function pointer types */
+      BLAST_SequenceBlk*, 
+      BlastQueryInfo* query_info,
+      BLAST_SequenceBlk*, 
+      BlastGapAlignStruct*, 
+      const BlastScoringParameters*,
+      const BlastExtensionParameters*, 
+      const BlastHitSavingParameters*,
+      BlastInitHitList*, 
+      BlastHSPList**, 
+      BlastGappedStats*);
      
 /** Word finder function pointer type */
 typedef Int2 (*BlastWordFinderType) 
-     (BLAST_SequenceBlk*, BLAST_SequenceBlk*,
-      LookupTableWrap*, Int4**, const BlastInitialWordParameters*,
-      Blast_ExtendWord*, Uint4*, Uint4*, Int4, BlastInitHitList*,
+     (BLAST_SequenceBlk*, /**< @todo comment function pointer types */
+      BLAST_SequenceBlk*,
+      LookupTableWrap*, 
+      Int4**, 
+      const BlastInitialWordParameters*,
+      Blast_ExtendWord*, 
+      BlastOffsetPair*,
+      Int4, 
+      BlastInitHitList*,
       BlastUngappedStats*);
 
 #ifdef __cplusplus
