@@ -1,4 +1,4 @@
-/* $Id: blast_filter.h,v 1.23 2004/09/13 12:40:35 madden Exp $
+/* $Id: blast_filter.h,v 1.28 2005/03/02 13:52:59 madden Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -37,11 +37,33 @@
 
 #include <algo/blast/core/blast_def.h>
 #include <algo/blast/core/blast_message.h>
+#include <algo/blast/core/blast_options.h>
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+/** BLASTNA element used to mask bases in BLAST */
+extern const Uint1 kNuclMask;
+/** NCBISTDAA element used to mask residues in BLAST */
+extern const Uint1 kProtMask;
+
+/** Repeats filtering default options. */
+#define REPEATS_SEARCH_EVALUE 0.1       /**< Default e-value threshold */
+#define REPEATS_SEARCH_PENALTY -1       /**< Default mismatch penalty */
+#define REPEATS_SEARCH_GAP_OPEN 2       /**< Default gap opening cost */
+#define REPEATS_SEARCH_GAP_EXTEND 1     /**< Default gap extension cost */
+#define REPEATS_SEARCH_WORD_SIZE 11     /**< Default word size */
+#define REPEATS_SEARCH_XDROP_UNGAPPED 40/**< Default X-dropoff for ungapped 
+                                           extension */
+#define REPEATS_SEARCH_XDROP_FINAL 90   /**< Default X-dropoff for gapped 
+                                           extension with traceback */
+#define REPEATS_SEARCH_FILTER_STRING "F"/**< Default filter string - 
+                                           no filtering */
+
+/** Largest gap allowed to be filled between repeat mask intervals */
+#define REPEAT_MASK_LINK_VALUE 5
+
 /** Create and initialize a new sequence interval.
  * @param head existing BlastSeqLoc to add onto, if *head
  *   is NULL then it will be set to new BlastSeqLoc, may be NULL [in|out]
@@ -74,6 +96,23 @@ BlastMaskLoc* BlastMaskLocFree(BlastMaskLoc* mask_loc);
 NCBI_XBLAST_EXPORT
 BlastMaskLoc* BlastMaskLocNew(Int4 total);
 
+/** Given a BlastMaskLoc with an array of lists of DNA mask locations, 
+ * substitutes that array by a new array of per-protein-frame mask location 
+ * lists.
+ * @param mask_loc Mask locations structure [in|out]
+ * @param query_info Query information structure, containing contexts data [in]
+ */
+Int2 BlastMaskLocDNAToProtein(BlastMaskLoc* mask_loc, 
+                              const BlastQueryInfo* query_info);
+
+/** Given a BlastMaskLoc with an array of lists of mask locations per protein
+ * frame, recalculates all mask offsets in terms of the DNA sequence. 
+ * @param mask_loc Mask locations structure [in|out]
+ * @param query_info Query information structure, containing contexts data [in]
+ */
+Int2 BlastMaskLocProteinToDNA(BlastMaskLoc* mask_loc, 
+                              const BlastQueryInfo* query_info);
+
 /** Go through all mask locations in one sequence, 
  * combine any that overlap. Deallocate the memory for the locations that 
  * were on the list, produce a new (merged) list of locations. 
@@ -108,16 +147,16 @@ BLAST_ComplementMaskLocations(EBlastProgramType program_number,
    BlastQueryInfo* query_info, BlastMaskLoc* mask_loc, 
    BlastSeqLoc* *complement_mask);
 
-/** Runs filtering functions, according to the string "instructions", on the
+/** Runs filtering functions, according to the filtering options, returns
  * SeqLocPtr. Should combine all SeqLocs so they are non-redundant.
  * @param program_number Type of BLAST program [in]
  * @param sequence The sequence or part of the sequence to be filtered [in]
  * @param length Length of the (sub)sequence [in]
  * @param offset Offset into the full sequence [in]
- * @param instructions String of instructions to filtering functions. [in]
- * @param mask_at_hash If TRUE masking is done while making the lookup table
- *                     only. [out] 
+ * @param filter_options specifies how filtering is to be done [in]
  * @param seqloc_retval Resulting locations for filtered region. [out]
+ * @param blast_message error messages on error [out]
+ * @return zero on success
 */
 NCBI_XBLAST_EXPORT
 Int2
@@ -125,26 +164,24 @@ BlastSetUp_Filter(EBlastProgramType program_number,
     Uint1* sequence, 
     Int4 length, 
     Int4 offset, 
-    const char* instructions, 
-    Boolean *mask_at_hash, 
-    BlastSeqLoc* *seqloc_retval);
+    const SBlastFilterOptions* filter_options,
+    BlastSeqLoc* *seqloc_retval,
+    Blast_Message * *blast_message);
 
 
 /** Does preparation for filtering and then calls BlastSetUp_Filter
  * @param query_blk sequence to be filtered [in]
  * @param query_info info on sequence to be filtered [in]
  * @param program_number one of blastn,blastp,blastx,etc. [in]
- * @param filter_string instructions for filtering [in]
+ * @param filter_options specifies how filtering is to be done [in]
  * @param filter_out resulting locations for filtered region. [out]
- * @param mask_at_hash If TRUE masking is done while making the lookup table
- *                     only. [out] 
  * @param blast_message message that needs to be sent back to user.
 */
 NCBI_XBLAST_EXPORT
 Int2
 BlastSetUp_GetFilteringLocations(BLAST_SequenceBlk* query_blk, BlastQueryInfo* query_info,
-    EBlastProgramType program_number, const char* filter_string, BlastMaskLoc** filter_out, Boolean* mask_at_hash,
-    Blast_Message* *blast_message);
+    EBlastProgramType program_number, const SBlastFilterOptions* filter_options, 
+    BlastMaskLoc** filter_out, Blast_Message* *blast_message);
 
 /** Masks the letters in buffer.
  * This is a low-level routine and takes a raw buffer which it assumes
@@ -173,6 +210,18 @@ Int2
 BlastSetUp_MaskQuery(BLAST_SequenceBlk* query_blk, BlastQueryInfo* query_info,
     BlastMaskLoc *filter_maskloc, EBlastProgramType program_number);
 
+/** Produces SBlastFilterOptions from a string that has been traditionally supported
+ * in blast.
+ * @param program_number Type of BLAST program [in]
+ * @param instructions the string describing the filtering to be done [in]
+ * @param filtering_options the structure to be filled in [out]
+ * @param blast_message optional field for error messages [out]
+ * @return zero on success
+ */
+NCBI_XBLAST_EXPORT
+Int2
+BlastFilteringOptionsFromString(EBlastProgramType program_number, const char* instructions, 
+    SBlastFilterOptions* *filtering_options, Blast_Message* *blast_message);
 
 #ifdef __cplusplus
 }
