@@ -1,4 +1,4 @@
-/*  $Id: blast_stat.h,v 1.57 2004/10/05 21:33:23 camacho Exp $
+/*  $Id: blast_stat.h,v 1.67 2005/04/27 19:49:17 dondosha Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -57,24 +57,24 @@ extern "C" {
   Structure to hold the Karlin-Altschul parameters.
 */
 typedef struct Blast_KarlinBlk {
-		double	Lambda; /**< Lambda value used in statistics */
-		double	K; /**< K value used in statistics */
-		double	logK; /**< natural log of K value used in statistics */
-		double	H; /**< H value used in statistics */
-		double	paramC;	/**< for use in seed. */
-	} Blast_KarlinBlk;
+      double   Lambda; /**< Lambda value used in statistics */
+      double   K; /**< K value used in statistics */
+      double   logK; /**< natural log of K value used in statistics */
+      double   H; /**< H value used in statistics */
+      double   paramC;  /**< for use in seed. */
+   } Blast_KarlinBlk;
 
 
 
 
 /********************************************************************
 
-	Structures relating to scoring or the BlastScoreBlk
+   Structures relating to scoring or the BlastScoreBlk
 
 ********************************************************************/
 
-#define BLAST_SCORE_MIN	INT2_MIN   /**< minimum allowed score (for one letter comparison). */
-#define BLAST_SCORE_MAX	INT2_MAX   /**< maximum allowed score (for one letter comparison). */
+#define BLAST_SCORE_MIN INT2_MIN   /**< minimum allowed score (for one letter comparison). */
+#define BLAST_SCORE_MAX INT2_MAX   /**< maximum allowed score (for one letter comparison). */
 
 
 /** Holds score frequencies used in calculation
@@ -90,54 +90,76 @@ typedef struct Blast_ScoreFreq {
     double*      sprob;     /**< arrays for frequency of given score, shifted down by score_min. */
 } Blast_ScoreFreq;
 
-#define BLAST_MATRIX_SIZE 32
+/** Scoring matrix used in BLAST */
+typedef struct SBlastScoreMatrix {
+    int** data;         /**< actual scoring matrix data, stored in row-major
+                          form */
+    size_t ncols;       /**< number of columns */
+    size_t nrows;       /**< number of rows */
+} SBlastScoreMatrix;
 
-/* Remove me */
-typedef struct SBLASTMatrixStructure {
-    Int4 *matrix[BLAST_MATRIX_SIZE];
-    Int4 long_matrix[BLAST_MATRIX_SIZE*BLAST_MATRIX_SIZE]; /* not used */
-} SBLASTMatrixStructure;
+/** Scoring matrix data used in PSI-BLAST */
+typedef struct SPsiBlastScoreMatrix {
+    SBlastScoreMatrix* pssm;    /**< position-specific score matrix */
+    double** freq_ratios;       /**< PSSM's frequency ratios, dimensions are
+                                  specified in pssm data above */
+    Blast_KarlinBlk* kbp;       /**< Karlin-Altschul block associated with this
+                                  PSSM */
+} SPsiBlastScoreMatrix;
+
+/** Allocates a new SPsiBlastScoreMatrix structure of dimensions ncols by
+ * BLASTAA_SIZE.
+ * @param ncols number of columns (i.e.: query length) [in]
+ * @return NULL in case of memory allocation failure, else new
+ * SPsiBlastScoreMatrix structure
+ */
+SPsiBlastScoreMatrix*
+SPsiBlastScoreMatrixNew(size_t ncols);
+
+/** Deallocates a SPsiBlastScoreMatrix structure.
+ * @param matrix structure to deallocate [in]
+ * @return NULL
+ */
+SPsiBlastScoreMatrix*
+SPsiBlastScoreMatrixFree(SPsiBlastScoreMatrix* matrix);
 
 /** Structure used for scoring calculations.
 */
 typedef struct BlastScoreBlk {
-	Boolean		protein_alphabet; /**< TRUE if alphabet_code is for a 
+   Boolean     protein_alphabet; /**< TRUE if alphabet_code is for a 
 protein alphabet (e.g., ncbistdaa etc.), FALSE for nt. alphabets. */
-	Uint1		alphabet_code;	/**< NCBI alphabet code. */
-	Int2 		alphabet_size;  /**< size of alphabet. */
-	Int2 		alphabet_start;  /**< numerical value of 1st letter. */
-	SBLASTMatrixStructure* matrix_struct;	/**< Holds info about matrix. */
-	Int4 **matrix;  /**< Substitution matrix */
-	Int4 **posMatrix;  /**< Sub matrix for position depend BLAST. */
-	Int2	mat_dim1;	/**< dimensions of matrix. */
-	Int2	mat_dim2;	/**< dimensions of matrix. */
-	Int4	loscore;   /**< Min.  substitution scores */
-	Int4	hiscore;   /**< Max. substitution scores */
-	Int4	penalty;   /**< penalty for mismatch in blastn. */
-	Int4	reward;    /**< reward for match in blastn. */
+   Uint1    alphabet_code; /**< NCBI alphabet code. */
+   Int2     alphabet_size;  /**< size of alphabet. */
+   Int2     alphabet_start;  /**< numerical value of 1st letter. */
+   char*    name;           /**< name of scoring matrix. */
+   ListNode*   comments;    /**< Comments about scoring matrix. */
+   SBlastScoreMatrix* matrix;   /**< scoring matrix data */
+   SPsiBlastScoreMatrix* psi_matrix;    /**< PSSM and associated data. If this
+                                         is not NULL, then the BLAST search is
+                                         position specific (i.e.: PSI-BLAST) */
+   Int4  loscore;   /**< Min.  substitution scores */
+   Int4  hiscore;   /**< Max. substitution scores */
+   Int4  penalty;   /**< penalty for mismatch in blastn. */
+   Int4  reward;    /**< reward for match in blastn. */
         double  scale_factor; /**< multiplier for all cutoff and dropoff scores */
-	Boolean		read_in_matrix; /**< If TRUE, matrix is read in, otherwise
-					produce one from penalty and reward above. */
-	Blast_ScoreFreq** sfp;	/**< score frequencies. */
-	double **posFreqs; /**<matrix of position specific frequencies*/
-	/* kbp & kbp_gap are ptrs that should be set to kbp_std, kbp_psi, etc. */
-	Blast_KarlinBlk** kbp; 	/**< Karlin-Altschul parameters. Actually just a placeholder. */
-	Blast_KarlinBlk** kbp_gap; /**< K-A parameters for gapped alignments.  Actually just a placeholder. */
-	/* Below are the Karlin-Altschul parameters for non-position based ('std')
-	and position based ('psi') searches. */
-	Blast_KarlinBlk **kbp_std,  /**< K-A parameters for ungapped alignments	*/
-                    **kbp_psi,	    /**< K-A parameters for position-based alignments. */
+   Boolean     read_in_matrix; /**< If TRUE, matrix is read in, otherwise
+               produce one from penalty and reward above. @todo should this be
+                an allowed way of specifying the matrix to use? */
+   Blast_ScoreFreq** sfp;  /**< score frequencies for scoring matrix. */
+   /* kbp & kbp_gap are ptrs that should be set to kbp_std, kbp_psi, etc. */
+   Blast_KarlinBlk** kbp;  /**< Karlin-Altschul parameters. Actually just a placeholder. */
+   Blast_KarlinBlk** kbp_gap; /**< K-A parameters for gapped alignments.  Actually just a placeholder. */
+   /* Below are the Karlin-Altschul parameters for non-position based ('std')
+   and position based ('psi') searches. */
+   Blast_KarlinBlk **kbp_std,  /**< K-A parameters for ungapped alignments */
+                    **kbp_psi,       /**< K-A parameters for position-based alignments. */
                     **kbp_gap_std,  /**< K-A parameters for std (not position-based) alignments */
                     **kbp_gap_psi;  /**< K-A parameters for psi alignments. */
-	Blast_KarlinBlk* 	kbp_ideal;	/**< Ideal values (for query with average database composition). */
-	Int4 number_of_contexts;	/**< Used by sfp and kbp, how large are these*/
-	char* 	name;		/**< name of matrix. */
-	Uint1* 	ambiguous_res;	/**< Array of ambiguous res. (e.g, 'X', 'N')*/
-	Int2		ambig_size,	/**< size of array above. FIXME: not needed here? */
-			ambig_occupy;	/**< How many occupied? */
-	ListNode*	comments;	/**< Comments about matrix. */
-	Int4    	query_length;   /**< the length of the query. */
-	Int8	effective_search_sp;	/**< product of above two */
+   Blast_KarlinBlk*  kbp_ideal;  /**< Ideal values (for query with average database composition). */
+   Int4 number_of_contexts;   /**< Used by sfp and kbp, how large are these*/
+   Uint1*   ambiguous_res; /**< Array of ambiguous res. (e.g, 'X', 'N')*/
+   Int2     ambig_size, /**< size of array above. FIXME: not needed here? */
+         ambig_occupy;  /**< How many occupied? */
 } BlastScoreBlk;
 
 /** 
@@ -145,7 +167,7 @@ Stores the letter frequency of a sequence or database.
 */
 typedef struct Blast_ResFreq {
     Uint1   alphabet_code;    /**< indicates alphabet. */
-    double* prob;	      /**< letter probs, (possible) non-zero offset. */
+    double* prob;       /**< letter probs, (possible) non-zero offset. */
     double* prob0;            /**< probs, zero offset. */
 } Blast_ResFreq;
 
@@ -163,13 +185,6 @@ BlastScoreBlk* BlastScoreBlkNew (Uint1 alphabet, Int4 number_of_contexts);
  */
 BlastScoreBlk* BlastScoreBlkFree (BlastScoreBlk* sbp);
 
-/**  Sets sbp->matrix field using sbp->name field using
- * the matrices in the toolkit (util/tables/raw_scoremat.h).
- * @param sbp the object containing matrix and name [in|out]
- * @return 0 on success, 1 if matrix could not be loaded 
- */
-Int2 BlastScoreBlkMatrixLoad(BlastScoreBlk* sbp);
-
 /** Set the ambiguous residue (e.g, 'N', 'X') in the BlastScoreBlk*.
  *  Convert from ncbieaa to sbp->alphabet_code (i.e., ncbistdaa) first.
  *
@@ -180,35 +195,49 @@ Int2 BlastScoreBlkMatrixLoad(BlastScoreBlk* sbp);
 Int2 BLAST_ScoreSetAmbigRes (BlastScoreBlk* sbp, char ambiguous_res);
 
 
-/** Calculate the ungapped Karlin parameters. This function should be called 
- * once for each context, or frame translated.
- * @param sbp the object to be modified [in|out]
- * @param string the query sequence [in]
- * @param length length of above sequence [in]
- * @param context_number which element in various arrays [in]
- * @return zero on success.
+/** Calculate and fill the ungapped Karlin-Altschul parameters in the
+ * BlastScoreBlk structure.
+ * @param program BLAST program type, needed to decide whether to substitute
+ *                ideal values. [in]
+ * @param sbp Scoring block to work with [in] [out]
+ * @param query Buffer containing (concatenated) query sequence [in]
+ * @param query_info Information about offsets of concatenated queries [in]
+ * @return 0 if ungapped Karlin-Altschul parameters could be 
+ *               calculated for at least one context; 1 otherwise.
  */
-Int2 BLAST_ScoreBlkFill (BlastScoreBlk* sbp, char* string, Int4 length, Int4 context_number);
- 
+Int2
+Blast_ScoreBlkKbpUngappedCalc(EBlastProgramType program, 
+                              BlastScoreBlk* sbp, Uint1* query, 
+                              BlastQueryInfo* query_info);
+
 /** This function fills in the BlastScoreBlk structure.  
  * Tasks are:
- *	-read in the matrix
- *	-set maxscore
+ * -read in the matrix
+ * -set maxscore
  * @param sbp Scoring block [in] [out]
  * @param matrix Full path to the matrix in the directory structure [in]
 */
-Int2 BLAST_ScoreBlkMatFill (BlastScoreBlk* sbp, char* matrix);
+Int2 Blast_ScoreBlkMatrixFill (BlastScoreBlk* sbp, char* matrix);
  
 /** Callocs a Blast_KarlinBlk
  * @return pointer to the Blast_KarlinBlk
 */
-Blast_KarlinBlk* Blast_KarlinBlkCreate (void);
+Blast_KarlinBlk* Blast_KarlinBlkNew (void);
+
+/** Copies contents of one Karlin block to another. Both must be allocated
+ * before this function is called.
+ * @param kbp_to Karlin block to copy values to [in] [out]
+ * @param kbp_from Karlin block to copy values from [in]
+ * @return 0 on success; -1 if either argument is NULL on input.
+ */ 
+Int2 Blast_KarlinBlkCopy(Blast_KarlinBlk* kbp_to, Blast_KarlinBlk* kbp_from);
+
 
 /** Deallocates the KarlinBlk
  * @param kbp KarlinBlk to be deallocated [in]
  * @return NULL
 */
-Blast_KarlinBlk* Blast_KarlinBlkDestruct(Blast_KarlinBlk* kbp);
+Blast_KarlinBlk* Blast_KarlinBlkFree(Blast_KarlinBlk* kbp);
 
 /** Fills in lambda, H, and K values, as calculated by Stephen Altschul 
  *  in Methods in Enzy. (vol 266, page 474).
@@ -225,26 +254,15 @@ Int2 Blast_KarlinBlkGappedCalc (Blast_KarlinBlk* kbp, Int4 gap_open,
         Blast_Message** error_return);
 
 
-/** Calculates the standard Karlin parameters.  This is used
- *       if the query is translated and the calculated (real) Karlin
- *       parameters are bad, as they're calculated for non-coding regions.
- * @param sbp ScoreBlk used to calculate "ideal" values. [in]
+/** Calculates the Karlin-Altschul parameters assuming standard residue
+ * compositions for the query and subject sequences. It populates the kbp_ideal
+ * field of its sbp argument. This is used if the query is translated and the
+ * calculated (real) Karlin parameters are bad, as they're calculated for 
+ * non-coding regions.
+ * @param sbp ScoreBlk used to calculate "ideal" values. [in|out]
+ * @return 0 on success, 1 on failure
 */
-Blast_KarlinBlk* Blast_KarlinBlkIdealCalc(const BlastScoreBlk* sbp);
-
-
-/** Fills KarlinBlk pointers in BlastScoreBlk with "ideal" values if the 
- * ideal Lambda is less than the actual Lambda.  This happens if 
- * if the query is translated and the calculated (real) Karlin
- *
- * parameters are bad, as they're calculated for non-coding regions.
- * @param sbp the object to be modified [in|out]
- * @param context_start first context to start with [in]
- * @param context_end last context to work on [in]
- * @return zero on success
- */
-Int2 Blast_KarlinBlkStandardCalc(BlastScoreBlk* sbp, Int4 context_start, 
-                                 Int4 context_end);
+Int2 Blast_ScoreBlkKbpIdealCalc(BlastScoreBlk* sbp);
 
 /** Attempts to fill KarlinBlk for given gap opening, extensions etc.
  *
@@ -257,16 +275,16 @@ Int2 Blast_KarlinBlkStandardCalc(BlastScoreBlk* sbp, Int4 context_start,
  *          1 if matrix not found
  *           2 if matrix found, but open, extend etc. values not supported.
 */
-Int2 Blast_KarlinkGapBlkFill(Blast_KarlinBlk* kbp, Int4 gap_open, Int4 gap_extend, Int4 decline_align, const char* matrix_name);
+Int2 Blast_KarlinBlkGappedLoadFromTables(Blast_KarlinBlk* kbp, Int4 gap_open, Int4 gap_extend, Int4 decline_align, const char* matrix_name);
 
-/** Prints a messages about the allowed matrices, BlastKarlinkGapBlkFill should return 1 before this is called. 
+/** Prints a messages about the allowed matrices, BlastKarlinBlkGappedFill should return 1 before this is called. 
  * @param matrix the matrix to print a message about [in]
  * @return the message
  */
 char* BLAST_PrintMatrixMessage(const char *matrix);
 
 /** Prints a messages about the allowed open etc values for the given matrix, 
- * BlastKarlinkGapBlkFill should return 2 before this is called. 
+ * BlastKarlinBlkGappedFill should return 2 before this is called. 
  * @param matrix name of the matrix [in]
  * @param gap_open gap existence cost [in]
  * @param gap_extend cost to extend a gap by one [in]
@@ -366,7 +384,6 @@ double BLAST_UnevenGapSumE (Int4 query_start_points, Int4 subject_start_points,
 
 /** Calculates the e-value if a collection of distinct alignments with
  *   arbitrarily large gaps between the alignments
- * @param kbp statistical parameters [in]
  * @param num number of distinct alignments in the collection [in]
  * @param xsum the sum of the scores of these alignments each individually
  *     normalized using an appropriate value of Lambda and logK [in]
@@ -395,22 +412,23 @@ void BLAST_GetAlphaBeta (const char* matrixName, double *alpha,
                     double *beta, Boolean gapped, Int4 gap_open, Int4 gap_extend);
 
 
-/** Calculate a new PSSM, using composition-based statistics, for use
+/** Rescale the PSSM, using composition-based statistics, for use
  *  with RPS BLAST. This function produces a PSSM for a single RPS DB
  *  sequence (of size db_seq_length) and incorporates information from 
  *  the RPS blast query. Each individual database sequence must call this
  *  function to retrieve its own PSSM. The matrix is returned (and must
  *  be freed elsewhere). posMatrix is the portion of the complete 
  *  concatenated PSSM that is specific to this DB sequence 
+ * @todo revise to use existing code
  * @param scalingFactor used to rescale Lambda [in]
  * @param rps_query_length length of query sequence [in]
  * @param rps_query_seq the query sequence [in]
  * @param db_seq_length Length of the database sequence [in]
  * @param posMatrix matrix (actual) values to be used [in]
  * @param matrix_name Name of the score matrix underlying the RPS search [in]
- * @return new pssm 
+ * @return rescaled pssm 
  */
-Int4 ** RPSCalculatePSSM(double scalingFactor, Int4 rps_query_length, 
+Int4 ** RPSRescalePssm(double scalingFactor, Int4 rps_query_length, 
                    const Uint1 * rps_query_seq, Int4 db_seq_length, 
                    Int4 **posMatrix, const char *matrix_name);
 
@@ -474,7 +492,7 @@ Blast_ResFreq* Blast_ResFreqNew(const BlastScoreBlk* sbp);
 /** Deallocates Blast_ResFreq and prob0 element.
  * @param rfp the Blast_ResFreq to be deallocated.
 */
-Blast_ResFreq* Blast_ResFreqDestruct(Blast_ResFreq* rfp);
+Blast_ResFreq* Blast_ResFreqFree(Blast_ResFreq* rfp);
 
 
 /** Calculates residues frequencies given a standard distribution.
@@ -498,7 +516,7 @@ Blast_ScoreFreqNew(Int4 score_min, Int4 score_max);
  * @return NULL
  */
 Blast_ScoreFreq*
-Blast_ScoreFreqDestruct(Blast_ScoreFreq* sfp);
+Blast_ScoreFreqFree(Blast_ScoreFreq* sfp);
 
 /** Fills a buffer with the 'standard' alphabet 
  * (given by STD_AMINO_ACID_FREQS[index].ch).
@@ -520,7 +538,7 @@ Blast_GetStdAlphabet(Uint1 alphabet_code, Uint1* residues,
  * @return zero on success, 1 on error.
  */
 Int2
-Blast_KarlinBlkCalc(Blast_KarlinBlk* kbp, Blast_ScoreFreq* sfp);
+Blast_KarlinBlkUngappedCalc(Blast_KarlinBlk* kbp, Blast_ScoreFreq* sfp);
 
 /**  Given a sequence of 'length' amino acid residues, compute the
  *   probability of each residue and put that in the array resProb
