@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   10/21/98
 *
-* $Revision: 6.33 $
+* $Revision: 6.34 $
 *
 * File Description:  New GenBank flatfile generator, private header
 *
@@ -220,8 +220,8 @@ typedef struct asn2gb_job {
 
 typedef void (*Asn2gbWriteFunc) (CharPtr str, Pointer userdata, BlockType blocktype);
 
-typedef SeqAnnotPtr (*Asn2gbLockFunc) (SeqIdPtr sip, Pointer remotedata);
-typedef void (*Asn2gbFreeFunc) (SeqAnnotPtr sap, Pointer remotedata);
+typedef ValNodePtr (*Asn2gbLockFunc) (SeqIdPtr sip, Pointer remotedata);
+typedef void (*Asn2gbFreeFunc) (ValNodePtr vnp, Pointer remotedata);
 
 typedef struct XtraData {
   Asn2gbWriteFunc  ffwrite;
