@@ -29,7 +29,7 @@
 *   
 * Version Creation Date: 2/2/94
 *
-* $Revision: 6.13 $
+* $Revision: 6.15 $
 *
 * File Description:  Sequence editing utilities
 *
@@ -39,6 +39,14 @@
 * -------  ----------  -----------------------------------------------------
 *
 * $Log: edutil.h,v $
+* Revision 6.15  2005/04/28 20:10:32  bollin
+* added new function AdjustFeaturesForInsertion which is called by BioseqInsert
+* and also by a new function in sequin3.c for converting a raw bioseq to a delta
+* and inserting gaps
+*
+* Revision 6.14  2005/04/06 19:33:15  bollin
+* made it possible to insert and remove gaps from delta sequences
+*
 * Revision 6.13  2004/10/08 16:04:16  bollin
 * added ability to check when an action will remove a feature
 *
@@ -225,6 +233,19 @@ NLM_EXTERN BioseqPtr LIBCALL BioseqCopyEx (SeqIdPtr newid, BioseqPtr oldbsp, Int
 NLM_EXTERN SeqLocPtr LIBCALL SeqLocCopyPart PROTO((SeqLocPtr the_segs, Int4 from, Int4 to,
          Uint1 strand, Boolean group, Int2Ptr first_segp, Int2Ptr last_segp));
 
+/* This function is used by BioseqInsert to adjust features on a sequence after
+ * an insertion.  pos indicates the start of the insertion, len indicates
+ * the length of the inserted sequence, and do_split indicates whether features
+ * that cover the insertion should be split at either side of the insertion.
+ */
+extern void 
+AdjustFeaturesForInsertion 
+(BioseqPtr tobsp, 
+ SeqIdPtr  to_id,
+ Int4 pos, 
+ Int4 len, 
+ Boolean do_split);
+
 /*****************************************************************************
 *
 * BioseqInsert (from_id, from, to, strand, to_id, pos, from_feat, to_feat,
@@ -570,6 +591,7 @@ NLM_EXTERN DeltaSeqPtr LIBCALL GappedSeqLocsToDeltaSeqs(SeqLocPtr slp);
  * in desktop/seqpanel.c
  */
 typedef enum { eSeqEdInsert, eSeqEdDelete, eSeqEdFeatMove, eSeqEdTranslate,
+               eSeqEdInsertGap, eSeqEdDeleteGap,
                eSeqEdJournalStart, eSeqEdJournalEnd } ESeqEdJournalAction ;
 
 typedef struct seqedjournaldata
@@ -594,6 +616,10 @@ typedef struct seqedjournaldata
   ValNodePtr          affected_feats;      /* This is a list of features which were shortened by
                                             * an eSeqEdDelete operation - their locations will
                                             * need to be reconstructed if the operation is undone. */
+  Boolean             unknown_gap;         /* This is used only when action is eSeqEdInsertGap
+                                            * or eSeqEdDeleteGap.  It indicates whether the gap
+                                            * being inserted (or deleted) from a delta sequence
+                                            * is unknown. */                                            
   BioseqPtr           bsp;                 /* The Bioseq for which the action is to be/was applied. */
   Uint2               moltype;             /* Molecule type for bsp.  Stored for convenience. */
   Uint2               entityID;            /* entityID for bsp.  Stored for convenience. */
