@@ -29,7 +29,7 @@
 *   
 * Version Creation Date: 6/30/98
 *
-* $Revision: 6.48 $
+* $Revision: 6.50 $
 *
 * File Description:  Reengineered and optimized exploration functions
 *                      to be used for future code
@@ -133,6 +133,17 @@ typedef struct seqmgrfeatcontext {
   Uint4         index;
 } SeqMgrFeatContext, PNTR SeqMgrFeatContextPtr;
 
+typedef struct seqmgrandcontext {
+  Uint2         entityID;
+  Uint4         itemID;
+  AnnotDescPtr  adp;
+  Uint1         annotdesctype;
+  Pointer       userdata;
+                          /* the following fields are for internal use only */
+  Pointer       omdp;
+  Uint4         index;
+} SeqMgrAndContext, PNTR SeqMgrAndContextPtr;
+
 /*****************************************************************************
 *
 *   SeqMgrIndexFeatures builds indices of sorted features for all bioseqs in an
@@ -278,6 +289,13 @@ NLM_EXTERN SeqFeatPtr LIBCALL SeqMgrGetNextFeature (
   SeqMgrFeatContext PNTR context
 );
 
+NLM_EXTERN AnnotDescPtr LIBCALL SeqMgrGetNextAnnotDesc (
+  BioseqPtr bsp,
+  AnnotDescPtr curr,
+  Uint1 annotDescChoice,
+  SeqMgrAndContext PNTR context
+);
+
 /*****************************************************************************
 *
 *   Callback types for SeqMgrExploreBioseqs, SeqMgrExploreSegments,
@@ -425,6 +443,13 @@ NLM_EXTERN SeqFeatPtr LIBCALL SeqMgrGetDesiredFeature (
   SeqMgrFeatContext PNTR context
 );
 
+NLM_EXTERN AnnotDescPtr LIBCALL SeqMgrGetDesiredAnnotDesc (
+  Uint2 entityID,
+  BioseqPtr bsp,
+  Uint4 itemID,
+  SeqMgrAndContext PNTR context
+);
+
 /*****************************************************************************
 *
 *   SeqMgrVisitDescriptors and SeqMgrVisitFeatures visit all descriptors or
@@ -587,6 +612,20 @@ NLM_EXTERN Uint2 LIBCALL SeqMgrIndexFeaturesEx (
   Boolean dorevfeats
 );
 
+/*****************************************************************************
+*
+*   SeqMgrIndexFeaturesExEx allows indexing of remotely fetched features
+*
+*****************************************************************************/
+
+NLM_EXTERN Uint2 LIBCALL SeqMgrIndexFeaturesExEx (
+  Uint2 entityID,
+  Pointer ptr,
+  Boolean flip,
+  Boolean dorevfeats,
+  ValNodePtr extra
+);
+
 /*****************************************************************************
 *
 *   If indexed with dorevfeats TRUE, SeqMgrExploreFeaturesRev presents features
