@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   9/2/97
 *
-* $Revision: 6.107 $
+* $Revision: 6.118 $
 *
 * File Description: 
 *
@@ -48,6 +48,7 @@
 #include <ncbi.h>
 #include <sequtil.h>
 #include <objpubme.h>
+#include <objentgene.h>
 #include <util/creaders/alnread.h>
 
 #undef NLM_EXTERN
@@ -69,6 +70,9 @@ NLM_EXTERN SeqEntryPtr LIBCALL GetBestTopParentForDataEx (Uint2 entityID, Bioseq
 NLM_EXTERN SeqEntryPtr LIBCALL GetBestTopParentForItemIDEx (Uint2 entityID, Uint2 itemID, Uint2 itemtype, Boolean skipGenProdSet);
 
 NLM_EXTERN SeqIdPtr SeqIdFindWorst (SeqIdPtr sip);
+NLM_EXTERN void ChangeSeqIdToWorstID (SeqIdPtr sip);
+NLM_EXTERN void ChangeSeqLocToWorstID (SeqLocPtr slp);
+
 NLM_EXTERN SeqIdPtr MakeSeqID (CharPtr str);
 NLM_EXTERN SeqIdPtr MakeUniqueSeqID (CharPtr prefix);
 
@@ -119,6 +123,8 @@ NLM_EXTERN void        ResetSeqFeatInterval (SeqFeatPtr sfp);
 NLM_EXTERN void        AddSeqFeatInterval (SeqFeatPtr sfp, BioseqPtr bsp, Int4 from, Int4 to,
                                        Boolean partial5, Boolean partial3);
 
+NLM_EXTERN void        AddSeqLocPoint (SeqLocPtr PNTR old_slp, SeqIdPtr sip, Int4 location,
+                                       Boolean fuzz_before, Boolean fuzz_after, Int2 strand);
 NLM_EXTERN void        AddSeqFeatPoint (SeqFeatPtr sfp, BioseqPtr bsp, Int4 location, Boolean fuzz_before, Boolean fuzz_after, Int2 strand);
 
 /* AddSeqEntryToSeqEntry and ReplaceSeqEntryWithSeqEntry leave
@@ -314,9 +320,10 @@ NLM_EXTERN Boolean SerialNumberInString (CharPtr str);
 NLM_EXTERN int LIBCALLBACK SortVnpByString (VoidPtr ptr1, VoidPtr ptr2);
 NLM_EXTERN ValNodePtr UniqueValNode (ValNodePtr list);
 
-/* for sorting valnode list by data.intvalue */
+/* for sorting and uniquing valnode list by data.intvalue */
 
 NLM_EXTERN int LIBCALLBACK SortByIntvalue (VoidPtr ptr1, VoidPtr ptr2);
+NLM_EXTERN ValNodePtr UniqueIntValNode (ValNodePtr list);
 
 /* keytag sorts/uniques and then owns valnode character list */
 
@@ -393,6 +400,11 @@ NLM_EXTERN Pointer ReadAsnFastaOrFlatFile (FILE *fp, Uint2Ptr datatypeptr, Uint2
                                            Boolean forceNuc, Boolean forceProt,
                                            Boolean parseFastaSeqId, Boolean fastaAsSimpleSeq);
 
+/* ReadDeltaFasta reads a FASTA file, combining raw sequence and >?unk100 lines into
+a delta Bioseq.  The file pointer stops at the next FASTA with a real SeqID. */
+
+NLM_EXTERN BioseqPtr ReadDeltaFasta (FILE *fp, Uint2Ptr entityIDptr);
+
 /* PromoteXrefs expands generef or protref feature cross-references (made by reading a
 feature table with ReadAsnFastaOrFlatFile) to stand-alone gene features or protein features
 and protein bioseqs.  It processes ALL features in the list - you give it the FIRST sfp. */
@@ -430,6 +442,10 @@ require reindexing) */
 
 NLM_EXTERN void BasicSeqEntryCleanup (SeqEntryPtr sep);
 
+/* CleanUpSeqFeat componenet of BasicSeqEntryCleanup, can be called for external features */
+
+NLM_EXTERN void CleanUpSeqFeat (SeqFeatPtr sfp, Boolean isEmblOrDdbj, Boolean stripSerial, ValNodePtr PNTR publist);
+
 /* CautiousSeqEntryCleanup is a gradual consolidation and replacement of functions in SeriousSeqEntryCleanup,
 which does change the itemID structure, and is intended to be safe for a retrofit of the ID database */
 
@@ -570,6 +586,23 @@ NLM_EXTERN Int4 VisitBioSourcesInSep (SeqEntryPtr sep, Pointer userdata, VisitBi
 typedef void (*ScanBioseqSetFunc) (SeqEntryPtr sep, Pointer userdata);
 NLM_EXTERN Int4 ScanBioseqSetRelease (CharPtr inputFile, Boolean binary, Boolean compressed, Pointer userdata, ScanBioseqSetFunc callback);
 
+/* function to scan binary ASN.1 file of entrezgene release as Entrezgene-Set */
+
+typedef void (*ScanEntrezgeneSetFunc) (EntrezgenePtr egp, Pointer userdata);
+NLM_EXTERN Int4 ScanEntrezgeneSetRelease (CharPtr inputFile, Boolean binary, Boolean compressed, Pointer userdata, ScanEntrezgeneSetFunc callback);
+
+/* general file recursion function - directory must not be empty, proc callback function must not be NULL */
+
+typedef void (*DirExpProc) (CharPtr filename, Pointer userdata);
+
+NLM_EXTERN Int4 DirExplore (
+  CharPtr directory,
+  CharPtr filter,
+  CharPtr suffix,
+  DirExpProc proc,
+  Pointer userdata
+);
+
 /* PubMed registered fetch functionality */
 
 NLM_EXTERN PubmedEntryPtr LIBCALL GetPubMedForUid (Int4 uid);
@@ -599,6 +632,7 @@ typedef struct readbuffer {
 
 extern void FreeBufferedReadList (ValNodePtr vnp);
 
+extern CharPtr AlignmentStringToSequenceString (CharPtr aln_str, Uint1 moltype);
 extern SeqEntryPtr MakeSequinDataFromAlignment (TAlignmentFilePtr afp, Uint1 moltype);
 extern SeqEntryPtr MakeSequinDataFromAlignmentEx (TAlignmentFilePtr afp, Uint1 moltype, Boolean check_ids);
 extern SeqEntryPtr make_seqentry_for_seqentry (SeqEntryPtr sep);
@@ -606,6 +640,9 @@ extern void ProcessPseudoMiscFeatsForEntityID (Uint2 entityID);
 extern void ConvertPseudoCDSToMiscFeatsForEntityID (Uint2 entityID);
 
 extern SeqAlignPtr FindAlignmentsForBioseq (BioseqPtr bsp);
+extern ValNodePtr FindAlignSeqAnnotsForBioseq (BioseqPtr bsp);
+extern Boolean IsSequenceFirstInPairwise (SeqEntryPtr sep, SeqIdPtr sip);
+extern Boolean RemoveSequenceFromAlignments (SeqEntryPtr sep, SeqIdPtr sip);
 
 #ifdef __cplusplus
 }
