@@ -9,8 +9,8 @@
 #include <asn.h>
 #endif
 
-static char * asnfilename = "asnfeat.h69";
-static AsnValxNode avnx[192] = {
+static char * asnfilename = "asnfeat.h13";
+static AsnValxNode avnx[201] = {
     {2,NULL,0,0.0,NULL } ,
     {20,"other" ,0,0.0,&avnx[2] } ,
     {20,"family" ,1,0.0,&avnx[3] } ,
@@ -102,109 +102,118 @@ static AsnValxNode avnx[192] = {
     {20,"signal-peptide" ,23,0.0,&avnx[89] } ,
     {20,"transit-peptide" ,24,0.0,&avnx[90] } ,
     {20,"transmembrane-region" ,25,0.0,&avnx[91] } ,
+    {20,"nitrosylation" ,26,0.0,&avnx[92] } ,
     {20,"other" ,255,0.0,NULL } ,
-    {20,"unknown" ,0,0.0,&avnx[93] } ,
-    {20,"pol1" ,1,0.0,&avnx[94] } ,
-    {20,"pol2" ,2,0.0,&avnx[95] } ,
-    {20,"pol3" ,3,0.0,&avnx[96] } ,
-    {20,"bacterial" ,4,0.0,&avnx[97] } ,
-    {20,"viral" ,5,0.0,&avnx[98] } ,
-    {20,"rna" ,6,0.0,&avnx[99] } ,
-    {20,"organelle" ,7,0.0,&avnx[100] } ,
+    {20,"unknown" ,0,0.0,&avnx[94] } ,
+    {20,"pol1" ,1,0.0,&avnx[95] } ,
+    {20,"pol2" ,2,0.0,&avnx[96] } ,
+    {20,"pol3" ,3,0.0,&avnx[97] } ,
+    {20,"bacterial" ,4,0.0,&avnx[98] } ,
+    {20,"viral" ,5,0.0,&avnx[99] } ,
+    {20,"rna" ,6,0.0,&avnx[100] } ,
+    {20,"organelle" ,7,0.0,&avnx[101] } ,
     {20,"other" ,255,0.0,NULL } ,
     {2,NULL,0,0.0,NULL } ,
     {2,NULL,0,0.0,NULL } ,
-    {20,"unknown" ,0,0.0,&avnx[104] } ,
-    {20,"single" ,1,0.0,&avnx[105] } ,
-    {20,"multiple" ,2,0.0,&avnx[106] } ,
+    {20,"unknown" ,0,0.0,&avnx[105] } ,
+    {20,"single" ,1,0.0,&avnx[106] } ,
+    {20,"multiple" ,2,0.0,&avnx[107] } ,
     {20,"region" ,3,0.0,NULL } ,
-    {20,"unknown" ,0,0.0,&avnx[108] } ,
-    {20,"rna-seq" ,1,0.0,&avnx[109] } ,
-    {20,"rna-size" ,2,0.0,&avnx[110] } ,
-    {20,"np-map" ,3,0.0,&avnx[111] } ,
-    {20,"np-size" ,4,0.0,&avnx[112] } ,
-    {20,"pe-seq" ,5,0.0,&avnx[113] } ,
-    {20,"cDNA-seq" ,6,0.0,&avnx[114] } ,
-    {20,"pe-map" ,7,0.0,&avnx[115] } ,
-    {20,"pe-size" ,8,0.0,&avnx[116] } ,
-    {20,"pseudo-seq" ,9,0.0,&avnx[117] } ,
-    {20,"rev-pe-map" ,10,0.0,&avnx[118] } ,
+    {20,"unknown" ,0,0.0,&avnx[109] } ,
+    {20,"rna-seq" ,1,0.0,&avnx[110] } ,
+    {20,"rna-size" ,2,0.0,&avnx[111] } ,
+    {20,"np-map" ,3,0.0,&avnx[112] } ,
+    {20,"np-size" ,4,0.0,&avnx[113] } ,
+    {20,"pe-seq" ,5,0.0,&avnx[114] } ,
+    {20,"cDNA-seq" ,6,0.0,&avnx[115] } ,
+    {20,"pe-map" ,7,0.0,&avnx[116] } ,
+    {20,"pe-size" ,8,0.0,&avnx[117] } ,
+    {20,"pseudo-seq" ,9,0.0,&avnx[118] } ,
+    {20,"rev-pe-map" ,10,0.0,&avnx[119] } ,
     {20,"other" ,255,0.0,NULL } ,
-    {20,"unknown" ,0,0.0,&avnx[120] } ,
-    {20,"physiological" ,1,0.0,&avnx[121] } ,
-    {20,"in-vitro" ,2,0.0,&avnx[122] } ,
-    {20,"oocyte" ,3,0.0,&avnx[123] } ,
-    {20,"transfection" ,4,0.0,&avnx[124] } ,
-    {20,"transgenic" ,5,0.0,&avnx[125] } ,
+    {20,"unknown" ,0,0.0,&avnx[121] } ,
+    {20,"physiological" ,1,0.0,&avnx[122] } ,
+    {20,"in-vitro" ,2,0.0,&avnx[123] } ,
+    {20,"oocyte" ,3,0.0,&avnx[124] } ,
+    {20,"transfection" ,4,0.0,&avnx[125] } ,
+    {20,"transgenic" ,5,0.0,&avnx[126] } ,
     {20,"other" ,255,0.0,NULL } ,
     {3,NULL,1,0.0,NULL } ,
     {2,NULL,0,0.0,NULL } ,
     {2,NULL,0,0.0,NULL } ,
-    {20,"helix" ,1,0.0,&avnx[130] } ,
-    {20,"sheet" ,2,0.0,&avnx[131] } ,
+    {20,"helix" ,1,0.0,&avnx[131] } ,
+    {20,"sheet" ,2,0.0,&avnx[132] } ,
     {20,"turn" ,3,0.0,NULL } ,
-    {20,"unknown" ,0,0.0,&avnx[133] } ,
-    {20,"genomic" ,1,0.0,&avnx[134] } ,
-    {20,"chloroplast" ,2,0.0,&avnx[135] } ,
-    {20,"chromoplast" ,3,0.0,&avnx[136] } ,
-    {20,"kinetoplast" ,4,0.0,&avnx[137] } ,
-    {20,"mitochondrion" ,5,0.0,&avnx[138] } ,
-    {20,"plastid" ,6,0.0,&avnx[139] } ,
-    {20,"macronuclear" ,7,0.0,&avnx[140] } ,
-    {20,"extrachrom" ,8,0.0,&avnx[141] } ,
-    {20,"plasmid" ,9,0.0,&avnx[142] } ,
-    {20,"transposon" ,10,0.0,&avnx[143] } ,
-    {20,"insertion-seq" ,11,0.0,&avnx[144] } ,
-    {20,"cyanelle" ,12,0.0,&avnx[145] } ,
-    {20,"proviral" ,13,0.0,&avnx[146] } ,
-    {20,"virion" ,14,0.0,&avnx[147] } ,
-    {20,"nucleomorph" ,15,0.0,&avnx[148] } ,
-    {20,"apicoplast" ,16,0.0,&avnx[149] } ,
-    {20,"leucoplast" ,17,0.0,&avnx[150] } ,
-    {20,"proplastid" ,18,0.0,&avnx[151] } ,
+    {20,"unknown" ,0,0.0,&avnx[134] } ,
+    {20,"genomic" ,1,0.0,&avnx[135] } ,
+    {20,"chloroplast" ,2,0.0,&avnx[136] } ,
+    {20,"chromoplast" ,3,0.0,&avnx[137] } ,
+    {20,"kinetoplast" ,4,0.0,&avnx[138] } ,
+    {20,"mitochondrion" ,5,0.0,&avnx[139] } ,
+    {20,"plastid" ,6,0.0,&avnx[140] } ,
+    {20,"macronuclear" ,7,0.0,&avnx[141] } ,
+    {20,"extrachrom" ,8,0.0,&avnx[142] } ,
+    {20,"plasmid" ,9,0.0,&avnx[143] } ,
+    {20,"transposon" ,10,0.0,&avnx[144] } ,
+    {20,"insertion-seq" ,11,0.0,&avnx[145] } ,
+    {20,"cyanelle" ,12,0.0,&avnx[146] } ,
+    {20,"proviral" ,13,0.0,&avnx[147] } ,
+    {20,"virion" ,14,0.0,&avnx[148] } ,
+    {20,"nucleomorph" ,15,0.0,&avnx[149] } ,
+    {20,"apicoplast" ,16,0.0,&avnx[150] } ,
+    {20,"leucoplast" ,17,0.0,&avnx[151] } ,
+    {20,"proplastid" ,18,0.0,&avnx[152] } ,
     {20,"endogenous-virus" ,19,0.0,NULL } ,
     {3,NULL,0,0.0,NULL } ,
-    {20,"unknown" ,0,0.0,&avnx[154] } ,
-    {20,"natural" ,1,0.0,&avnx[155] } ,
-    {20,"natmut" ,2,0.0,&avnx[156] } ,
-    {20,"mut" ,3,0.0,&avnx[157] } ,
-    {20,"artificial" ,4,0.0,&avnx[158] } ,
-    {20,"synthetic" ,5,0.0,&avnx[159] } ,
+    {20,"unknown" ,0,0.0,&avnx[155] } ,
+    {20,"natural" ,1,0.0,&avnx[156] } ,
+    {20,"natmut" ,2,0.0,&avnx[157] } ,
+    {20,"mut" ,3,0.0,&avnx[158] } ,
+    {20,"artificial" ,4,0.0,&avnx[159] } ,
+    {20,"synthetic" ,5,0.0,&avnx[160] } ,
     {20,"other" ,255,0.0,NULL } ,
     {3,NULL,0,0.0,NULL } ,
-    {20,"chromosome" ,1,0.0,&avnx[162] } ,
-    {20,"map" ,2,0.0,&avnx[163] } ,
-    {20,"clone" ,3,0.0,&avnx[164] } ,
-    {20,"subclone" ,4,0.0,&avnx[165] } ,
-    {20,"haplotype" ,5,0.0,&avnx[166] } ,
-    {20,"genotype" ,6,0.0,&avnx[167] } ,
-    {20,"sex" ,7,0.0,&avnx[168] } ,
-    {20,"cell-line" ,8,0.0,&avnx[169] } ,
-    {20,"cell-type" ,9,0.0,&avnx[170] } ,
-    {20,"tissue-type" ,10,0.0,&avnx[171] } ,
-    {20,"clone-lib" ,11,0.0,&avnx[172] } ,
-    {20,"dev-stage" ,12,0.0,&avnx[173] } ,
-    {20,"frequency" ,13,0.0,&avnx[174] } ,
-    {20,"germline" ,14,0.0,&avnx[175] } ,
-    {20,"rearranged" ,15,0.0,&avnx[176] } ,
-    {20,"lab-host" ,16,0.0,&avnx[177] } ,
-    {20,"pop-variant" ,17,0.0,&avnx[178] } ,
-    {20,"tissue-lib" ,18,0.0,&avnx[179] } ,
-    {20,"plasmid-name" ,19,0.0,&avnx[180] } ,
-    {20,"transposon-name" ,20,0.0,&avnx[181] } ,
-    {20,"insertion-seq-name" ,21,0.0,&avnx[182] } ,
-    {20,"plastid-name" ,22,0.0,&avnx[183] } ,
-    {20,"country" ,23,0.0,&avnx[184] } ,
-    {20,"segment" ,24,0.0,&avnx[185] } ,
-    {20,"endogenous-virus-name" ,25,0.0,&avnx[186] } ,
-    {20,"transgenic" ,26,0.0,&avnx[187] } ,
-    {20,"environmental-sample" ,27,0.0,&avnx[188] } ,
-    {20,"isolation-source" ,28,0.0,&avnx[189] } ,
+    {20,"chromosome" ,1,0.0,&avnx[163] } ,
+    {20,"map" ,2,0.0,&avnx[164] } ,
+    {20,"clone" ,3,0.0,&avnx[165] } ,
+    {20,"subclone" ,4,0.0,&avnx[166] } ,
+    {20,"haplotype" ,5,0.0,&avnx[167] } ,
+    {20,"genotype" ,6,0.0,&avnx[168] } ,
+    {20,"sex" ,7,0.0,&avnx[169] } ,
+    {20,"cell-line" ,8,0.0,&avnx[170] } ,
+    {20,"cell-type" ,9,0.0,&avnx[171] } ,
+    {20,"tissue-type" ,10,0.0,&avnx[172] } ,
+    {20,"clone-lib" ,11,0.0,&avnx[173] } ,
+    {20,"dev-stage" ,12,0.0,&avnx[174] } ,
+    {20,"frequency" ,13,0.0,&avnx[175] } ,
+    {20,"germline" ,14,0.0,&avnx[176] } ,
+    {20,"rearranged" ,15,0.0,&avnx[177] } ,
+    {20,"lab-host" ,16,0.0,&avnx[178] } ,
+    {20,"pop-variant" ,17,0.0,&avnx[179] } ,
+    {20,"tissue-lib" ,18,0.0,&avnx[180] } ,
+    {20,"plasmid-name" ,19,0.0,&avnx[181] } ,
+    {20,"transposon-name" ,20,0.0,&avnx[182] } ,
+    {20,"insertion-seq-name" ,21,0.0,&avnx[183] } ,
+    {20,"plastid-name" ,22,0.0,&avnx[184] } ,
+    {20,"country" ,23,0.0,&avnx[185] } ,
+    {20,"segment" ,24,0.0,&avnx[186] } ,
+    {20,"endogenous-virus-name" ,25,0.0,&avnx[187] } ,
+    {20,"transgenic" ,26,0.0,&avnx[188] } ,
+    {20,"environmental-sample" ,27,0.0,&avnx[189] } ,
+    {20,"isolation-source" ,28,0.0,&avnx[190] } ,
+    {20,"lat-lon" ,29,0.0,&avnx[191] } ,
+    {20,"collection-date" ,30,0.0,&avnx[192] } ,
+    {20,"collected-by" ,31,0.0,&avnx[193] } ,
+    {20,"identified-by" ,32,0.0,&avnx[194] } ,
+    {20,"fwd-primer-seq" ,33,0.0,&avnx[195] } ,
+    {20,"rev-primer-seq" ,34,0.0,&avnx[196] } ,
+    {20,"fwd-primer-name" ,35,0.0,&avnx[197] } ,
+    {20,"rev-primer-name" ,36,0.0,&avnx[198] } ,
     {20,"other" ,255,0.0,NULL } ,
-    {20,"experimental" ,1,0.0,&avnx[191] } ,
+    {20,"experimental" ,1,0.0,&avnx[200] } ,
     {20,"not-experimental" ,2,0.0,NULL } };
 
-static AsnType atx[234] = {
+static AsnType atx[238] = {
   {401, "Seq-feat" ,1,0,0,0,0,1,0,0,NULL,&atx[31],&atx[1],0,&atx[2]} ,
   {0, "id" ,128,0,0,1,0,0,0,0,NULL,&atx[2],NULL,0,&atx[12]} ,
   {402, "Feat-id" ,1,0,0,0,0,1,0,0,NULL,&atx[11],&atx[3],0,&atx[85]} ,
@@ -303,7 +312,7 @@ static AsnType atx[234] = {
   {0, "sncbistdaa" ,128,7,0,0,0,0,0,0,NULL,&atx[91],NULL,0,NULL} ,
   {0, "code-break" ,128,6,0,1,0,0,0,0,NULL,&atx[56],&atx[97],0,&atx[105]} ,
   {0, NULL,1,-1,0,0,0,0,0,0,NULL,&atx[98],NULL,0,NULL} ,
-  {425, "Code-break" ,1,0,0,0,0,0,0,0,NULL,&atx[31],&atx[99],0,&atx[232]} ,
+  {425, "Code-break" ,1,0,0,0,0,0,0,0,NULL,&atx[31],&atx[99],0,&atx[236]} ,
   {0, "loc" ,128,0,0,0,0,0,0,0,NULL,&atx[100],NULL,0,&atx[101]} ,
   {409, "Seq-loc" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[6]} ,
   {0, "aa" ,128,1,0,0,0,0,0,0,NULL,&atx[11],&atx[102],0,NULL} ,
@@ -379,37 +388,37 @@ static AsnType atx[234] = {
   {0, "rna" ,128,4,0,1,0,0,0,0,NULL,&atx[56],&atx[172],0,&atx[173]} ,
   {0, NULL,1,-1,0,0,0,0,0,0,NULL,&atx[18],NULL,0,NULL} ,
   {0, "expression" ,128,5,0,1,0,0,0,0,NULL,&atx[18],NULL,0,&atx[174]} ,
-  {0, "txsystem" ,128,6,0,0,0,0,0,0,NULL,&atx[80],&avnx[92],0,&atx[175]} ,
+  {0, "txsystem" ,128,6,0,0,0,0,0,0,NULL,&atx[80],&avnx[93],0,&atx[175]} ,
   {0, "txdescr" ,128,7,0,1,0,0,0,0,NULL,&atx[18],NULL,0,&atx[176]} ,
   {0, "txorg" ,128,8,0,1,0,0,0,0,NULL,&atx[177],NULL,0,&atx[178]} ,
   {404, "Org-ref" ,1,0,0,0,0,0,1,0,NULL,&atx[34],NULL,0,&atx[183]} ,
-  {0, "mapping-precise" ,128,9,0,0,1,0,0,0,&avnx[101],&atx[23],NULL,0,&atx[179]} ,
-  {0, "location-accurate" ,128,10,0,0,1,0,0,0,&avnx[102],&atx[23],NULL,0,&atx[180]} ,
-  {0, "inittype" ,128,11,0,1,0,0,0,0,NULL,&atx[80],&avnx[103],0,&atx[181]} ,
+  {0, "mapping-precise" ,128,9,0,0,1,0,0,0,&avnx[102],&atx[23],NULL,0,&atx[179]} ,
+  {0, "location-accurate" ,128,10,0,0,1,0,0,0,&avnx[103],&atx[23],NULL,0,&atx[180]} ,
+  {0, "inittype" ,128,11,0,1,0,0,0,0,NULL,&atx[80],&avnx[104],0,&atx[181]} ,
   {0, "evidence" ,128,12,0,1,0,0,0,0,NULL,&atx[27],&atx[182],0,NULL} ,
   {0, NULL,1,-1,0,0,0,0,0,0,NULL,&atx[183],NULL,0,NULL} ,
   {405, "Tx-evidence" ,1,0,0,0,0,0,0,0,NULL,&atx[31],&atx[184],0,NULL} ,
-  {0, "exp-code" ,128,0,0,0,0,0,0,0,NULL,&atx[80],&avnx[107],0,&atx[185]} ,
-  {0, "expression-system" ,128,1,0,0,1,0,0,0,&avnx[126],&atx[80],&avnx[119],0,&atx[186]} ,
-  {0, "low-prec-data" ,128,2,0,0,1,0,0,0,&avnx[127],&atx[23],NULL,0,&atx[187]} ,
-  {0, "from-homolog" ,128,3,0,0,1,0,0,0,&avnx[128],&atx[23],NULL,0,NULL} ,
+  {0, "exp-code" ,128,0,0,0,0,0,0,0,NULL,&atx[80],&avnx[108],0,&atx[185]} ,
+  {0, "expression-system" ,128,1,0,0,1,0,0,0,&avnx[127],&atx[80],&avnx[120],0,&atx[186]} ,
+  {0, "low-prec-data" ,128,2,0,0,1,0,0,0,&avnx[128],&atx[23],NULL,0,&atx[187]} ,
+  {0, "from-homolog" ,128,3,0,0,1,0,0,0,&avnx[129],&atx[23],NULL,0,NULL} ,
   {0, "num" ,128,15,0,0,0,0,0,0,NULL,&atx[189],NULL,0,&atx[190]} ,
   {412, "Numbering" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[193]} ,
-  {0, "psec-str" ,128,16,0,0,0,0,0,0,NULL,&atx[80],&avnx[129],0,&atx[191]} ,
+  {0, "psec-str" ,128,16,0,0,0,0,0,0,NULL,&atx[80],&avnx[130],0,&atx[191]} ,
   {0, "non-std-residue" ,128,17,0,0,0,0,0,0,NULL,&atx[18],NULL,0,&atx[192]} ,
   {0, "het" ,128,18,0,0,0,0,0,0,NULL,&atx[193],NULL,0,&atx[194]} ,
   {413, "Heterogen" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[152]} ,
   {0, "biosrc" ,128,19,0,0,0,0,0,0,NULL,&atx[195],NULL,0,NULL} ,
   {407, "BioSource" ,1,0,0,0,0,0,1,0,NULL,&atx[196],NULL,0,&atx[121]} ,
   {401, "BioSource" ,1,0,0,0,0,1,0,0,NULL,&atx[31],&atx[197],0,&atx[200]} ,
-  {0, "genome" ,128,0,0,0,1,0,0,0,&avnx[152],&atx[4],&avnx[132],0,&atx[198]} ,
-  {0, "origin" ,128,1,0,0,1,0,0,0,&avnx[160],&atx[4],&avnx[153],0,&atx[199]} ,
+  {0, "genome" ,128,0,0,0,1,0,0,0,&avnx[153],&atx[4],&avnx[133],0,&atx[198]} ,
+  {0, "origin" ,128,1,0,0,1,0,0,0,&avnx[161],&atx[4],&avnx[154],0,&atx[199]} ,
   {0, "org" ,128,2,0,0,0,0,0,0,NULL,&atx[200],NULL,0,&atx[201]} ,
   {402, "Org-ref" ,1,0,0,0,0,0,1,0,NULL,&atx[34],NULL,0,&atx[203]} ,
   {0, "subtype" ,128,3,0,1,0,0,0,0,NULL,&atx[56],&atx[202],0,&atx[207]} ,
   {0, NULL,1,-1,0,0,0,0,0,0,NULL,&atx[203],NULL,0,NULL} ,
   {403, "SubSource" ,1,0,0,0,0,0,0,0,NULL,&atx[31],&atx[204],0,NULL} ,
-  {0, "subtype" ,128,0,0,0,0,0,0,0,NULL,&atx[4],&avnx[161],0,&atx[205]} ,
+  {0, "subtype" ,128,0,0,0,0,0,0,0,NULL,&atx[4],&avnx[162],0,&atx[205]} ,
   {0, "name" ,128,1,0,0,0,0,0,0,NULL,&atx[18],NULL,0,&atx[206]} ,
   {0, "attrib" ,128,2,0,1,0,0,0,0,NULL,&atx[18],NULL,0,NULL} ,
   {0, "is-focus" ,128,4,0,1,0,0,0,0,NULL,&atx[148],NULL,0,NULL} ,
@@ -427,7 +436,7 @@ static AsnType atx[234] = {
   {0, "ext" ,128,9,0,1,0,0,0,0,NULL,&atx[158],NULL,0,&atx[220]} ,
   {0, "cit" ,128,10,0,1,0,0,0,0,NULL,&atx[221],NULL,0,&atx[222]} ,
   {416, "Pub-set" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[8]} ,
-  {0, "exp-ev" ,128,11,0,1,0,0,0,0,NULL,&atx[80],&avnx[190],0,&atx[223]} ,
+  {0, "exp-ev" ,128,11,0,1,0,0,0,0,NULL,&atx[80],&avnx[199],0,&atx[223]} ,
   {0, "xref" ,128,12,0,1,0,0,0,0,NULL,&atx[27],&atx[224],0,&atx[228]} ,
   {0, NULL,1,-1,0,0,0,0,0,0,NULL,&atx[225],NULL,0,NULL} ,
   {422, "SeqFeatXref" ,1,0,0,0,0,0,0,0,NULL,&atx[31],&atx[226],0,&atx[77]} ,
@@ -436,12 +445,16 @@ static AsnType atx[234] = {
   {0, "dbxref" ,128,13,0,1,0,0,0,0,NULL,&atx[27],&atx[229],0,&atx[230]} ,
   {0, NULL,1,-1,0,0,0,0,0,0,NULL,&atx[10],NULL,0,NULL} ,
   {0, "pseudo" ,128,14,0,1,0,0,0,0,NULL,&atx[23],NULL,0,&atx[231]} ,
-  {0, "except-text" ,128,15,0,1,0,0,0,0,NULL,&atx[18],NULL,0,NULL} ,
-  {426, "Genetic-code-table" ,1,0,0,0,0,0,0,0,NULL,&atx[27],&atx[233],0,NULL} ,
+  {0, "except-text" ,128,15,0,1,0,0,0,0,NULL,&atx[18],NULL,0,&atx[232]} ,
+  {0, "ids" ,128,16,0,1,0,0,0,0,NULL,&atx[27],&atx[233],0,&atx[234]} ,
+  {0, NULL,1,-1,0,0,0,0,0,0,NULL,&atx[2],NULL,0,NULL} ,
+  {0, "exts" ,128,17,0,1,0,0,0,0,NULL,&atx[27],&atx[235],0,NULL} ,
+  {0, NULL,1,-1,0,0,0,0,0,0,NULL,&atx[158],NULL,0,NULL} ,
+  {426, "Genetic-code-table" ,1,0,0,0,0,0,0,0,NULL,&atx[27],&atx[237],0,NULL} ,
   {0, NULL,1,-1,0,0,0,0,0,0,NULL,&atx[85],NULL,0,NULL} };
 
 static AsnModule ampx[8] = {
-  { "NCBI-Seqfeat" , "asnfeat.h69",&atx[0],NULL,&ampx[1],0,0} ,
+  { "NCBI-Seqfeat" , "asnfeat.h13",&atx[0],NULL,&ampx[1],0,0} ,
   { "NCBI-Rsite" , NULL,&atx[153],NULL,&ampx[2],0,0} ,
   { "NCBI-RNA" , NULL,&atx[122],NULL,&ampx[3],0,0} ,
   { "NCBI-Gene" , NULL,&atx[16],NULL,&ampx[4],0,0} ,
@@ -482,6 +495,10 @@ static AsnModulePtr amp = ampx;
 #define SEQ_FEAT_dbxref_E &at[229]
 #define SEQ_FEAT_pseudo &at[230]
 #define SEQ_FEAT_except_text &at[231]
+#define SEQ_FEAT_ids &at[232]
+#define SEQ_FEAT_ids_E &at[233]
+#define SEQ_FEAT_exts &at[234]
+#define SEQ_FEAT_exts_E &at[235]
 
 #define FEAT_ID &at[2]
 #define FEAT_ID_gibb &at[3]
@@ -553,8 +570,8 @@ static AsnModulePtr amp = ampx;
 #define CODE_BREAK_aa_ncbi8aa &at[103]
 #define CODE_BREAK_aa_ncbistdaa &at[104]
 
-#define GENETIC_CODE_TABLE &at[232]
-#define GENETIC_CODE_TABLE_E &at[233]
+#define GENETIC_CODE_TABLE &at[236]
+#define GENETIC_CODE_TABLE_E &at[237]
 
 
 /**************************************************
