@@ -1,4 +1,4 @@
-/*  $Id: ncbi_connection.c,v 6.42 2004/05/26 16:00:06 lavr Exp $
+/*  $Id: ncbi_connection.c,v 6.44 2004/11/15 19:33:42 lavr Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -454,7 +454,7 @@ extern EIO_Status CONN_Write
     if (!n_written)
         return eIO_InvalidArg;
     *n_written = 0;
-    if (size && !buf)
+    if (size  &&  !buf)
         return eIO_InvalidArg;
     CONN_NOT_NULL(Write);
 
@@ -540,11 +540,13 @@ static EIO_Status s_CONN_Read
     }
 
     /* read data from the internal peek buffer, if any */
-    *n_read = peek
-        ? BUF_Peek(conn->buf, buf, size) : BUF_Read(conn->buf, buf, size);
-    if (*n_read == size)
-        return eIO_Success;
-    buf = (char*) buf + *n_read;
+    if ( size ) {
+        *n_read = peek
+            ? BUF_Peek(conn->buf, buf, size) : BUF_Read(conn->buf, buf, size);
+        if (*n_read == size)
+            return eIO_Success;
+        buf = (char*) buf + *n_read;
+    }
 
     /* read data from the connection */
     {{
@@ -554,7 +556,7 @@ static EIO_Status s_CONN_Read
                                  conn->r_timeout == kDefaultTimeout ?
                                  conn->meta.default_timeout : conn->r_timeout);
         *n_read += x_read;
-        if (peek  &&  x_read)  /* save data in the internal peek buffer */
+        if ( peek )  /* save data in the internal peek buffer */
             verify(BUF_Write(&conn->buf, buf, x_read));
     }}
 
@@ -615,7 +617,7 @@ extern EIO_Status CONN_Read
     if (!n_read)
         return eIO_InvalidArg;
     *n_read = 0;
-    if (size && !buf)
+    if (size  &&  !buf)
         return eIO_InvalidArg;
 
     CONN_NOT_NULL(Read);
@@ -657,63 +659,62 @@ extern EIO_Status CONN_ReadLine
  size_t* n_read
  )
 {
-    EIO_Status status = eIO_Success;
-    char       w[1024];
-    size_t     len;
+    EIO_Status  status = eIO_Success;
+    int/*bool*/ done = 0/*false*/;
+    size_t      len;
 
     if (!n_read)
         return eIO_InvalidArg;
     *n_read = 0;
-    if (size && !line)
+    if (size  &&  !line)
         return eIO_InvalidArg;
 
     CONN_NOT_NULL(ReadLine);
 
     /* perform open, if not opened yet */
-    if (conn->state != eCONN_Open  &&  (status = s_Open(conn)) != eIO_Success)
-        return status;
-    assert(conn->state == eCONN_Open  &&  conn->meta.list != 0);
+    if (conn->state != eCONN_Open)
+        status = s_Open(conn);
 
-    /* flush the unwritten output data (if any) */
-    if ( conn->meta.flush ) {
-        conn->meta.flush(conn->meta.c_flush,
-                         conn->r_timeout == kDefaultTimeout ?
-                         conn->meta.default_timeout : conn->r_timeout);
+    if (status == eIO_Success) {
+        assert(conn->state == eCONN_Open  &&  conn->meta.list != 0);
+        /* flush the unwritten output data (if any) */
+        if ( conn->meta.flush ) {
+            conn->meta.flush(conn->meta.c_flush,
+                             conn->r_timeout == kDefaultTimeout ?
+                             conn->meta.default_timeout : conn->r_timeout);
+        }
     }
 
     len = 0;
-    while (len < size) {
+    while (status == eIO_Success) {
         size_t i;
+        char   w[1024];
         size_t x_read = 0;
         size_t x_size = BUF_Size(conn->buf);
-        char*  buf = size - len < sizeof(w) ? w : &line[len];
-        if (x_size == 0 || x_size > sizeof(w))
+        char*  x_buf  = size - len < sizeof(w) ? w : line + len;
+        if (x_size == 0  ||  x_size > sizeof(w))
             x_size = sizeof(w);
-        status = s_CONN_Read(conn, buf, x_size, &x_read, 0);
-        for (i = 0; i < x_read; i++) {
-            if (buf == w)
-                line[len] = buf[i];
-            if (buf[i] == '\n') {
-                line[len] = '\0';
-                i++;
-                break;
-            } else if (++len >= size) {
+        status = s_CONN_Read(conn, x_buf, size ? x_size : 0, &x_read, 0);
+        for (i = 0; i < x_read  &&  len < size; i++) {
+            char c = x_buf[i];
+            if (c == '\n') {
+                done = 1/*true*/;
                 i++;
                 break;
             }
+            if (x_buf == w)
+                line[len] = c;
+            len++;
         }
-        if (i < x_read) {
-            if (!BUF_PushBack(&conn->buf, &buf[i], x_read - i))
-                status = eIO_Unknown;
+        if (i < x_read  &&  !BUF_PushBack(&conn->buf, x_buf + i, x_read - i))
+            status = eIO_Unknown;
+        if (done  ||  len >= size)
             break;
-        } else if (status != eIO_Success) {
-            if (len < size)
-                line[len] = '\0';
-            break;
-        }
     }
-
+    if (len < size)
+        line[len] = '\0';
     *n_read = len;
+
     return status;
 }
 
@@ -854,6 +855,12 @@ extern EIO_Status CONN_WaitAsync
 /*
  * --------------------------------------------------------------------------
  * $Log: ncbi_connection.c,v $
+ * Revision 6.44  2004/11/15 19:33:42  lavr
+ * Speed-up CONN_ReadLine()
+ *
+ * Revision 6.43  2004/11/15 17:39:26  lavr
+ * Fix CONN_ReadLine() to always perform connector's read method
+ *
  * Revision 6.42  2004/05/26 16:00:06  lavr
  * Minor status fixes in CONN_SetTimeout() and CONN_ReadLine()
  *
