@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   10/23/91
 *
-* $Revision: 6.21 $
+* $Revision: 6.29 $
 *
 * File Description: 
 *   	miscellaneous functions
@@ -43,6 +43,30 @@
 * 02-16-94 Epstein     Retired Gestalt functions and definitions
 *
 * $Log: ncbimisc.c,v $
+* Revision 6.29  2005/04/13 21:36:18  kans
+* restored mac ifdefs around Nlm_P2Cstr and Nlm_C2Pstr functions
+*
+* Revision 6.28  2005/04/13 21:28:44  kans
+* Nlm_CtoPstr and Nlm_PtoCstr always uses our own code so we are not dependent on a particular version of Carbon
+*
+* Revision 6.27  2005/04/13 18:03:21  rsmith
+* On OSX don't define C2Pstr/P2Cstr, just implement Nlm_CtoPstr.
+*
+* Revision 6.26  2004/03/25 15:37:24  lavr
+* Change UINT64_MAX into UINT8_MAX back for the toolkit to compile portably
+*
+* Revision 6.25  2004/03/24 18:48:49  lebedev
+* Use UINT64_MAX in divray_init function
+*
+* Revision 6.24  2003/12/03 02:10:23  kans
+* added defines missing from Mac OS 10.3 headers
+*
+* Revision 6.23  2002/11/06 21:25:10  ucko
+* Don't assume MIPS is IRIX, or HPPA is HP/UX; allow Linux too, for both.
+*
+* Revision 6.22  2002/06/13 16:14:07  kans
+* fix includes for OS_UNIX_DARWIN with WIN_MAC (EN)
+*
 * Revision 6.21  2001/04/05 03:03:28  juran
 * Defined our own C2PStr() and P2Cstr() for use with Carbon (which omits them).
 *
@@ -149,6 +173,16 @@
 #include <TextUtils.h>
 #endif
 
+/* Missing from /usr/include/gcc/darwin/3.3/machine/limits.h */
+#ifdef __MWERKS__
+#ifdef OS_UNIX_DARWIN
+#ifndef __CHAR_BIT__
+#define __CHAR_BIT__ 8
+#endif
+#endif
+#endif
+/* End missing from /usr/include/gcc/darwin/3.3/machine/limits.h */
+
 /*
 TRIPLE_MARK is the character inserted before the thousands, millions,
 billions, etc. digit positions.  Change TRIPLE_MARK to a period
@@ -1116,13 +1150,13 @@ NLM_EXTERN Nlm_Boolean LIBCALL NodeListDelete (ValNodePtr head, Nlm_Int2 item)
 *
 *****************************************************************************/
 
-#ifdef OS_MAC
+#if defined(OS_MAC) || defined(OS_UNIX_DARWIN)
 
-// 2001-03-22:  Joshua Juran
-// C2PStr() and P2CStr() do not exist in Carbon, so we roll our own.
-# if TARGET_API_MAC_CARBON
-static void C2PStr(char *ioStr)
+/* C2PStr() and P2CStr() may or may not exist in Carbon, so we now always roll our own. */
+
+void Nlm_CtoPstr (Nlm_CharPtr str)
 {
+    char *ioStr = (char *) str;
 	size_t len = strlen(ioStr);
 	if (len > 255) {
 		len = 255;
@@ -1131,32 +1165,17 @@ static void C2PStr(char *ioStr)
 	ioStr[0] = len;
 }
 
-static void P2CStr(StringPtr ioStr)
+void Nlm_PtoCstr (Nlm_CharPtr str)
 {
+    StringPtr ioStr = (StringPtr) str;
 	Byte len = ioStr[0];
 	memmove(ioStr, ioStr + 1, len);
 	ioStr[len] = '\0';
 }
-# endif
 
-/*  p_churchill 12/99 removed conditional compilation support for 
- *  THINKC and MPW
- */
-void Nlm_CtoPstr (Nlm_CharPtr str)
-
-{
-  C2PStr ((char *) str);
-}
-
-void Nlm_PtoCstr (Nlm_CharPtr str)
-
-{
-  P2CStr ((StringPtr) str);
-}
 #endif
 
 
-
 NLM_EXTERN Nlm_Uint2 Nlm_SwitchUint2 (Nlm_Uint2 value)
 
 {
@@ -1689,11 +1708,11 @@ NLM_EXTERN const Nlm_Char* Nlm_PlatformName(void)
   return "Cray";
 #  elif defined(PROC_CONVEX)
   return "Convex";
-#  elif defined(PROC_HPPA)
+#  elif defined(PROC_HPPA) && !defined(OS_UNIX_LINUX)
   return "HPUX";
 #  elif defined(OS_UNIX_NEXT)
   return "NEXT";
-#  elif defined(PROC_MIPS)
+#  elif defined(PROC_MIPS) && !defined(OS_UNIX_LINUX)
   return "SGI_MIPS";
 #  elif defined(OS_UNIX_ULTRIX)
   return "ULTRIX";
