@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   10/21/98
 *
-* $Revision: 6.69 $
+* $Revision: 6.72 $
 *
 * File Description:  New GenBank flatfile generator application
 *
@@ -50,7 +50,7 @@
 #include <explore.h>
 #include <asn2gnbp.h>
 
-#define ASN2GB_APP_VER "2.1"
+#define ASN2GB_APP_VER "2.2"
 
 CharPtr ASN2GB_APPLICATION = ASN2GB_APP_VER;
 
@@ -387,6 +387,7 @@ static void LookForGi (
       case SEQID_TPG :
       case SEQID_TPE :
       case SEQID_TPD :
+      case SEQID_GPIPE :
         tsip = (TextSeqIdPtr) sip->data.ptrvalue;
         if (tsip != NULL && hgp->accn!= NULL &&
             StringICmp (tsip->accession, hgp->accn) == 0) {
@@ -745,7 +746,7 @@ static Int2 HandleMultipleRecords (
 
 #ifdef OS_UNIX
   if (compressed) {
-    gzcatprog = getenv ("NCBI_UNCOMPRESS-BINARY");
+    gzcatprog = getenv ("NCBI_UNCOMPRESS_BINARY");
     if (gzcatprog != NULL) {
       sprintf (cmmd, "%s %s", gzcatprog, inputFile);
     } else {
@@ -1401,6 +1402,9 @@ Int2 Main (
       return 1;
     }
     xtra.aip = aip;
+    if ((Boolean) ((flags & PRODUCE_OLD_GBSEQ) != 0)) {
+      do_insdseq = FALSE;
+    }
     if (do_insdseq) {
       atp = AsnLinkType (NULL, AsnFind ("INSDSet"));
       xtra.atp = AsnLinkType (NULL, AsnFind ("INSDSet.E"));
