@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   1/22/95
 *
-* $Revision: 6.2 $
+* $Revision: 6.11 $
 *
 * File Description: 
 *
@@ -45,12 +45,23 @@
 #include <seqsub.h>
 #include <gather.h>
 
+static ENUM_ALIST(name_suffix_alist)
+  {" ",    0},
+  {"Jr.",  1},
+  {"Sr.",  2},
+  {"II",   3},
+  {"III",  4},
+  {"IV",   5},
+  {"V",    6},
+  {"VI",   7},
+END_ENUM_ALIST
+
 typedef struct contactpage {
   DIALOG_MESSAGE_BLOCK
   TexT            firstname;
   TexT            middleinit;
   TexT            lastname;
-  TexT            suffix;
+  PopuP           suffix;
   DialoG          affil;
   GrouP           contactGrp [3];
 } ContactPage, PNTR ContactPagePtr;
@@ -93,8 +104,7 @@ static void ContactInfoPtrToContactPage (DialoG d, Pointer data)
               SafeSetTitle (cpp->lastname, txt);
               MemFree (txt);
               txt = ExtractTagListColumn (str, 3);
-              SafeSetTitle (cpp->suffix, txt);
-              MemFree (txt);
+	      SafeSetValue (cpp->suffix, atoi(txt)+1);
               MemFree (str);
             }
           }
@@ -117,6 +127,9 @@ static Pointer ContactPageToContactInfoPtr (DialoG d)
   PersonIdPtr     pid;
   Char            str [128];
   CharPtr         txt;
+  Char            suffix [32];
+  Uint2           suffixVal;
+
 
   cip = NULL;
   cpp = (ContactPagePtr) GetObjectExtra (d);
@@ -144,10 +157,10 @@ static Pointer ContactPageToContactInfoPtr (DialoG d)
           StringCat (str, txt);
           StringCat (str, "\t");
           MemFree (txt);
-          txt = SaveStringFromText (cpp->suffix);
-          StringCat (str, txt);
+          suffixVal = GetValue (cpp->suffix);
+	  sprintf (suffix, "%d", (int) (suffixVal - 1));
+          StringCat (str, suffix);
           StringCat (str, "\n");
-          MemFree (txt);
           txt = StringSave (str);
           nsp = AuthorSpreadsheetStringToNameStdPtr (txt);
           MemFree (txt);
@@ -326,7 +339,10 @@ extern DialoG CreateContactDialog (GrouP h, CharPtr title)
     cpp->firstname = DialogText (g, "", 8, NULL);
     cpp->middleinit = DialogText (g, "", 4, NULL);
     cpp->lastname = DialogText (g, "", 9, NULL);
-    cpp->suffix = DialogText (g, "", 3, NULL);
+    cpp->suffix = PopupList (g, TRUE, NULL);
+    SetObjectExtra (cpp->suffix, cpp, NULL);
+    InitEnumPopup (cpp->suffix, name_suffix_alist, NULL);
+    SetEnumPopup (cpp->suffix, name_suffix_alist, 0);
 
     cpp->affil = CreateExtAffilDialog (k, NULL,
                                        &(cpp->contactGrp [1]),
@@ -341,12 +357,58 @@ extern DialoG CreateContactDialog (GrouP h, CharPtr title)
 typedef struct citsubpage {
   DIALOG_MESSAGE_BLOCK
   DialoG          authors;
+  TexT            consortium;
   DialoG          date;
   DialoG          affil;
   TexT            descr;
   GrouP           citsubGrp [5];
 } CitsubPage, PNTR CitsubPagePtr;
 
+static AuthListPtr AddConsortiumToAuthList (AuthListPtr alp, TexT consortium)
+
+{
+  AuthorPtr    ap;
+  ValNodePtr   names;
+  PersonIdPtr  pid;
+
+  if (TextHasNoText (consortium)) return alp;
+  if (alp == NULL) {
+    alp = AuthListNew ();
+    alp->choice = 1;
+  }
+  pid = PersonIdNew ();
+  if (pid == NULL) return NULL;
+  pid->choice = 5;
+  pid->data = SaveStringFromText (consortium);
+  ap = AuthorNew ();
+  if (ap == NULL) return NULL;
+  ap->name = pid;
+  names = ValNodeAdd (&(alp->names));
+  names->choice = 1;
+  names->data.ptrvalue = ap;
+  return alp;
+}
+
+static void AuthListToConsortium (AuthListPtr alp, TexT consortium)
+
+{
+  AuthorPtr    ap;
+  ValNodePtr   names;
+  PersonIdPtr  pid;
+  CharPtr      str;
+
+  if (alp == NULL || consortium == NULL) return;
+  if (alp->choice != 1) return;
+  for (names = alp->names; names != NULL; names = names->next) {
+    ap = names->data.ptrvalue;
+    if (ap == NULL) continue;
+    pid = ap->name;
+    if (pid == NULL || pid->choice != 5) continue;
+    str = (CharPtr) pid->data;
+    SafeSetTitle (consortium, str);
+  }
+}
+
 static void CitSubPtrToCitsubPage (DialoG d, Pointer data)
 
 {
@@ -366,6 +428,7 @@ static void CitSubPtrToCitsubPage (DialoG d, Pointer data)
     if (csp != NULL) {
       alp = csp->authors;
       PointerToDialog (cpp->authors, (Pointer) alp);
+      AuthListToConsortium (alp, cpp->consortium);
       if (alp != NULL) {
         PointerToDialog (cpp->affil, alp->affil);
       }
@@ -393,6 +456,7 @@ static Pointer CitsubPageToCitSubPtr (DialoG d)
     csp = CitSubNew ();
     if (csp != NULL) {
       alp = (AuthListPtr) DialogToPointer (cpp->authors);
+      alp = AddConsortiumToAuthList (alp, cpp->consortium);
       if (alp != NULL) {
         alp->affil = DialogToPointer (cpp->affil);
       }
@@ -497,7 +561,7 @@ static ValNodePtr TestCitsubDialog (DialoG d)
 }
 
 static CharPtr citsubTabs [] = {
-  "Names", "Affiliation", "Description", NULL, NULL
+  "Names", "Affiliation", "Contact", "Description", NULL, NULL
 };
 
 static void ChangeCitsubSubPage (VoidPtr data, Int2 newval, Int2 oldval)
@@ -524,6 +588,7 @@ extern DialoG CreateCitSubDialog (GrouP h, CharPtr title, CitSubPtr csp)
   GrouP          k;
   GrouP          m;
   GrouP          p;
+  GrouP          q;
   GrouP          s;
   DialoG         tbs;
 
@@ -549,41 +614,44 @@ extern DialoG CreateCitSubDialog (GrouP h, CharPtr title, CitSubPtr csp)
     m = HiddenGroup (s, -1, 0, NULL);
     SetGroupSpacing (m, 10, 10);
 
-    citsubTabs [3] = NULL;
+    citsubTabs [4] = NULL;
     if (csp != NULL) {
       if (csp->date != NULL) {
-        citsubTabs [3] = "Date";
+        citsubTabs [4] = "Date";
       } else if (csp->imp != NULL && csp->imp->date != NULL) {
-        citsubTabs [3] = "Date";
+        citsubTabs [4] = "Date";
       } else if (GetAppProperty ("InternalNcbiSequin") != NULL) {
-        citsubTabs [3] = "Date";
+        citsubTabs [4] = "Date";
       }
     }
     tbs = CreateFolderTabs (m, citsubTabs, 0, 0, 0,
                             PROGRAM_FOLDER_TAB,
                             ChangeCitsubSubPage, (Pointer) cpp);
-    citsubTabs [3] = NULL;
+    citsubTabs [4] = NULL;
     k = HiddenGroup (m, 0, 0, NULL);
 
     cpp->citsubGrp [0] = HiddenGroup (k, -1, 0, NULL);
     cpp->authors = CreateAuthorDialog (cpp->citsubGrp [0], 3, -1);
+    q = HiddenGroup (cpp->citsubGrp [0], 2, 0, NULL);
+    StaticPrompt (q, "Consortium", 0, stdLineHeight, programFont, 'l');
+    cpp->consortium = DialogText (q, "", 16, NULL);
 
     cpp->affil = CreateExtAffilDialog (k, NULL,
                                        &(cpp->citsubGrp [1]),
-                                       &(cpp->citsubGrp [4]));
+                                       &(cpp->citsubGrp [2]));
 
-    cpp->citsubGrp [2] = HiddenGroup (k, 0, 2, NULL);
-    StaticPrompt (cpp->citsubGrp [2], "Description", 0, 0, programFont, 'c');
-    cpp->descr = ScrollText (cpp->citsubGrp [2], 30, 4, programFont, TRUE, NULL);
-    Hide (cpp->citsubGrp [2]);
-
-    cpp->citsubGrp [3] = HiddenGroup (k, -1, 0, NULL);
-    cpp->date = CreateDateDialog (cpp->citsubGrp [3], NULL);
+    cpp->citsubGrp [3] = HiddenGroup (k, 0, 2, NULL);
+    StaticPrompt (cpp->citsubGrp [3], "Description", 0, 0, programFont, 'c');
+    cpp->descr = ScrollText (cpp->citsubGrp [3], 30, 4, programFont, TRUE, NULL);
     Hide (cpp->citsubGrp [3]);
 
+    cpp->citsubGrp [4] = HiddenGroup (k, -1, 0, NULL);
+    cpp->date = CreateDateDialog (cpp->citsubGrp [4], NULL);
+    Hide (cpp->citsubGrp [4]);
+
     AlignObjects (ALIGN_CENTER, (HANDLE) tbs, (HANDLE) cpp->authors,
-                  (HANDLE) cpp->citsubGrp [2], (HANDLE) cpp->date,
-                  (HANDLE) cpp->affil, NULL);
+                  (HANDLE) q, (HANDLE) cpp->citsubGrp [3],
+                  (HANDLE) cpp->date, (HANDLE) cpp->affil, NULL);
   }
 
   return (DialoG) p;
@@ -706,6 +774,7 @@ static Pointer SubmitPageToSubmitBlockPtr (DialoG d)
 {
   Char            ch;
   Int2            i;
+  CharPtr         os;
   CharPtr         ptr;
   SubmitBlockPtr  sbp;
   CharPtr         sequin_app_version;
@@ -745,7 +814,12 @@ static Pointer SubmitPageToSubmitBlockPtr (DialoG d)
       if (StringHasNoText (spp->tool)) {
         sequin_app_version = (CharPtr) GetAppProperty ("SequinAppVersion");
         if (sequin_app_version != NULL) {
-          sprintf (tmp, "Sequin %s", sequin_app_version);
+          os = GetOpSysString ();
+          if (os != NULL) {
+            sprintf (tmp, "Sequin %s - %s", sequin_app_version, os);
+          } else {
+            sprintf (tmp, "Sequin %s", sequin_app_version);
+          }
         } else {
           StringCpy (tmp, "Sequin");
         }
@@ -924,6 +998,7 @@ static void SubmitBlockPtrToSubmitForm (ForM f, Pointer data)
 static Pointer SubmitFormToSubmitBlockPtr (ForM f)
 
 {
+  CitSubPtr       csp;
   SubmitFormPtr   sbfp;
   SubmitBlockPtr  sbp;
 
@@ -937,6 +1012,10 @@ static Pointer SubmitFormToSubmitBlockPtr (ForM f)
       if (sbp->contact == NULL || sbp->cit == NULL) {
         sbp = SubmitBlockFree (sbp);
       }
+      csp = sbp->cit;
+      if (csp->date == NULL) {
+        csp->date = DateCurr ();
+      }
     }
   }
   return (Pointer) sbp;
@@ -1069,13 +1148,10 @@ static Boolean ImportSubmitForm (ForM f, CharPtr filename)
     switch (sbfp->currentPage) {
       case SUBMISSION_PAGE :
         return ReadSubmitBlock (f, filename);
-        break;
       case CONTACT_PAGE :
         return ImportDialog (sbfp->contact, filename);
-        break;
       case CITATION_PAGE :
         return ImportDialog (sbfp->citsub, filename);
-        break;
       default :
         break;
     }
@@ -1093,13 +1169,10 @@ static Boolean ExportSubmitForm (ForM f, CharPtr filename)
     switch (sbfp->currentPage) {
       case SUBMISSION_PAGE :
         return WriteSubmitBlock (f, filename);
-        break;
       case CONTACT_PAGE :
         return ExportDialog (sbfp->contact, filename);
-        break;
       case CITATION_PAGE :
         return ExportDialog (sbfp->citsub, filename);
-        break;
       default :
         break;
     }
@@ -1107,11 +1180,54 @@ static Boolean ExportSubmitForm (ForM f, CharPtr filename)
   return FALSE;
 }
 
+static void CopyContactToCitAuthors (SubmitFormPtr sbfp)
+
+{
+  AuthListPtr     alp;
+  AuthorPtr       ap;
+  ContactInfoPtr  cip;
+  CitsubPagePtr   cpp;
+  ValNodePtr      names;
+
+  if (sbfp == NULL) return;
+  cpp = (CitsubPagePtr) GetObjectExtra (sbfp->citsub);
+  if (cpp == NULL) return;
+  cip = (ContactInfoPtr) DialogToPointer (sbfp->contact);
+  if (cip == NULL) return;
+  ap = NULL;
+  if (cip->contact != NULL) {
+    ap = (AuthorPtr) AsnIoMemCopy (cip->contact,
+                                   (AsnReadFunc) AuthorAsnRead,
+                                   (AsnWriteFunc) AuthorAsnWrite);
+  }
+  ContactInfoFree (cip);
+  if (ap == NULL) return;
+  alp = AuthListNew ();
+  if (alp != NULL) {
+    alp->choice = 1;
+    names = ValNodeNew (NULL);
+    alp->choice = 1;
+    alp->names = names;
+    if (names != NULL) {
+      names->choice = 1;
+      names->data.ptrvalue = ap;
+    }
+    if (ap == NULL) {
+      alp = AuthListFree (alp);
+    }
+    if (alp != NULL) {
+       PointerToDialog (cpp->authors, (Pointer) alp);
+    }
+  }
+  alp = AuthListFree (alp);
+}
+
 static void SetSubmitBlockImportExportItems (SubmitFormPtr sbfp)
 
 {
-  IteM  exportItm;
-  IteM  importItm;
+  CitSubPtr  csp;
+  IteM       exportItm;
+  IteM       importItm;
 
   if (sbfp != NULL) {
     importItm = FindFormMenuItem ((BaseFormPtr) sbfp, VIB_MSG_IMPORT);
@@ -1130,6 +1246,11 @@ static void SetSubmitBlockImportExportItems (SubmitFormPtr sbfp)
         SafeEnable (exportItm);
         break;
       case CITATION_PAGE :
+        csp = (CitSubPtr) DialogToPointer (sbfp->citsub);
+        if (csp == NULL) {
+          CopyContactToCitAuthors (sbfp);
+        }
+        CitSubFree (csp);
         SafeSetTitle (importItm, "Import Citation...");
         SafeSetTitle (exportItm, "Export Citation...");
         SafeEnable (importItm);
@@ -1452,10 +1573,12 @@ static void SubmitBlockFormActnProc (ForM f)
 extern Int2 LIBCALLBACK SubmitBlockGenFunc (Pointer data)
 
 {
+  ObjMgrDataPtr     omdp;
   OMProcControlPtr  ompcp;
   OMUserDataPtr     omudp;
   SubmitFormPtr     sbfp;
-  SubmitBlockPtr    sbp;
+  SubmitBlockPtr    sbp = NULL;
+  SeqSubmitPtr      ssp;
   WindoW            w;
 
   ompcp = (OMProcControlPtr) data;
@@ -1467,11 +1590,19 @@ extern Int2 LIBCALLBACK SubmitBlockGenFunc (Pointer data)
       break;
     case OBJ_SEQSUB :
       break;
+    case OBJ_SEQSUB_CIT :
+      omdp = ObjMgrGetData (ompcp->input_entityID);
+      if (omdp != NULL && omdp->datatype == OBJ_SEQSUB) {
+        ssp = (SeqSubmitPtr) omdp->dataptr;
+        if (ssp != NULL && ssp->datatype == 1) {
+          sbp = ssp->sub;
+        }
+       }
+      break;
     case 0 :
       break;
     default :
       return OM_MSG_RET_ERROR;
-      break;
   }
   omudp = ItemAlreadyHasEditor (ompcp->input_entityID, ompcp->input_itemID,
                                 ompcp->input_itemtype, ompcp->proc->procid);
@@ -1493,6 +1624,9 @@ extern Int2 LIBCALLBACK SubmitBlockGenFunc (Pointer data)
     sbfp->input_itemtype = ompcp->input_itemtype;
     sbfp->this_itemtype = OBJ_SUBMIT_BLOCK;
     sbfp->this_subtype = 0;
+    if (ompcp->input_itemtype == OBJ_SEQSUB_CIT && ompcp->input_itemID == 1) {
+      sbfp->input_itemtype = OBJ_SUBMIT_BLOCK;
+    }
     sbfp->procid = ompcp->proc->procid;
     sbfp->proctype = ompcp->proc->proctype;
     sbfp->userkey = OMGetNextUserKey ();
@@ -1501,6 +1635,7 @@ extern Int2 LIBCALLBACK SubmitBlockGenFunc (Pointer data)
 
     if (omudp != NULL) {
       omudp->userdata.ptrvalue = (Pointer) sbfp;
+      omudp->messagefunc = StdVibrantEditorMsgFunc;
     }
     if (sbp != NULL) {
       PointerToForm (sbfp->form, (Pointer) sbp);
@@ -1511,19 +1646,6 @@ extern Int2 LIBCALLBACK SubmitBlockGenFunc (Pointer data)
   return OM_MSG_RET_DONE;
 }
 
-static ValNodePtr SetStringFromVnp (CharPtr str, ValNodePtr vnp,
-                                    size_t maxsize)
-
-{
-  if (vnp != NULL) {
-    StringNCpy_0 (str, vnp->data.ptrvalue, maxsize);
-    vnp = vnp->next;
-  } else if (str != NULL && maxsize > 0) {
-    str [0] = '\0';
-  }
-  return vnp;
-}
-
 static AuthListPtr NameToAuthList (CharPtr first, CharPtr last)
 
 {
