@@ -29,7 +29,7 @@
 #
 # Version Creation Date:   4/3/2003
 #
-# $Revision: 1.2 $
+# $Revision: 1.3 $
 #
 # File Description:
 #
@@ -44,6 +44,9 @@
 # Modifications log:
 #
 #   $Log: setrsrc.sh,v $
+#   Revision 1.3  2004/11/19 14:42:56  kans
+#   restored setting of header/source/resource files to ttxt TEXT - CodeWarrior bug prevents setting debug flag on imported files with no type
+#
 #   Revision 1.2  2004/09/29 18:35:39  kans
 #   removed setting of header/source/resource files to ttxt TEXT - not needed by current CodeWarrior projects
 #
@@ -58,6 +61,12 @@ export PATH
 output=
 #output=-t
 
+# set the types of the source files.
+find ~/ncbi \( -name '*.h' -or -name '*.hpp' -or -name '*.c' -or -name '*.cpp' -o -name '*.r' \) | \
+    xargs $output SetFile -c 'ttxt' -t 'TEXT' 
+find ~/ncbi_cxx \( -name '*.h' -or -name '*.hpp' -or -name '*.c' -or -name '*.cpp' -o -name '*.r' \) | \
+    xargs $output SetFile -c 'ttxt' -t 'TEXT' 
+
 # Metrowerks project files.
 find ~/ncbi  -path '*/link/macmet/*.mcp'  | \
     xargs $output SetFile -c 'CWIE' -t 'MMPr' 
