@@ -34,6 +34,51 @@
 *
 * RCS Modification History:
 * $Log: netblap3.c,v $
+* Revision 1.105  2004/01/27 20:53:12  dondosha
+* Value of no_traceback megablast option is now Uint1 instead of Boolean
+*
+* Revision 1.104  2003/01/13 18:08:26  bealer
+* - Replace nonstandard snprintf() function with strcpy/strcat/strlen.
+*
+* Revision 1.103  2003/01/10 21:45:06  bealer
+* - Modify to return errors from BLASTGetUidsFromQuery instead of logging them.
+*
+* Revision 1.102  2002/10/30 18:54:58  madden
+* NULL out SeqLoc for lower-case masking
+*
+* Revision 1.101  2002/10/29 14:36:53  madden
+* Fix problem with repeated error messages
+*
+* Revision 1.100  2002/10/24 16:41:23  merezhuk
+* update to the BlastBioseq, will return all Blast-error responses instead off the first one
+*
+* Revision 1.99  2002/08/08 20:50:39  madden
+* Remove SPLIT_BLAST macros
+*
+* Revision 1.98  2002/05/23 22:57:03  dondosha
+* Made GetResponsePtr external
+*
+* Revision 1.97  2002/05/15 16:32:51  madden
+* Make QueryIsProteinFromType extern
+*
+* Revision 1.96  2002/05/10 12:56:27  madden
+* Allow network version of matrix as input
+*
+* Revision 1.95  2002/04/24 17:59:03  dondosha
+* Added handling of parameters for database splitting and megablast with discontiguous words
+*
+* Revision 1.94  2002/04/03 16:29:48  dondosha
+* Added SPLIT_BLAST macro to allow simultaneous use in current and new system
+*
+* Revision 1.93  2002/03/01 19:19:45  dondosha
+* Previous change made in a wrong place
+*
+* Revision 1.92  2002/03/01 19:04:24  dondosha
+* Set window size option to 0 for Web Mega BLAST
+*
+* Revision 1.91  2002/02/20 21:45:17  madden
+* Add protection against truncated posFreq sequence
+*
 * Revision 1.90  2001/09/06 20:27:10  dondosha
 * threshold_first removed from options
 *
@@ -758,8 +803,14 @@ BlastOptionsToParameters (BLAST_OptionsBlkPtr options)
         parameters->smith_waterman = options->smith_waterman;
         parameters->is_megablast = options->is_megablast_search;
         parameters->query_lcase_mask = (ValNodePtr) options->query_lcase_mask;
-        parameters->endpoint_results = options->no_traceback;
+        parameters->endpoint_results = (Boolean) options->no_traceback;
         parameters->percent_identity = (FloatHi) options->perc_identity;
+        parameters->first_db_seq = options->first_db_seq;
+        parameters->final_db_seq = options->final_db_seq;
+        parameters->window_size = options->window_size;
+        parameters->mb_template_length = options->mb_template_length;
+        parameters->mb_disc_type = options->mb_disc_type;
+
 	return parameters;
 }
 
@@ -805,6 +856,10 @@ BlastNet3BlockDestruct(BlastNet3BlockPtr blnet)
 	/* The entire gilist is not copied, so this prevents an error. */
 	if (blnet->parameters)
 		blnet->parameters->gilist = NULL;
+
+        /* The following is not allocated here */
+    	blnet->parameters->query_lcase_mask = NULL;
+    
 	BlastParametersFree(blnet->parameters);
 	/* individual elements freed elsewhere. */
 	response = blnet->response;
@@ -851,7 +906,7 @@ BlastNet3BlockNew(CharPtr program, CharPtr dbname)
 	return retval;
 }
 
-static Boolean
+NLM_EXTERN Boolean LIBCALL
 QueryIsProteinFromType(Uint2 type)
 {
 	switch (type)
@@ -871,7 +926,7 @@ QueryIsProteinFromType(Uint2 type)
 	}
 }
 
-static BlastResponsePtr GetResponsePtr(BlastResponsePtr response, Nlm_Uint1 choice)
+BlastResponsePtr GetResponsePtr(BlastResponsePtr response, Nlm_Uint1 choice)
 
 {
 	while (response)
@@ -886,7 +941,6 @@ static BlastResponsePtr GetResponsePtr(BlastResponsePtr response, Nlm_Uint1 choi
 	return response;
 }
 
-
 NLM_EXTERN BlastDbinfoPtr LIBCALL
 BlastRequestDbInfo (BlastNet3Hptr bl3hp, CharPtr database, Boolean is_prot)
 
@@ -907,7 +961,8 @@ BlastRequestDbInfo (BlastNet3Hptr bl3hp, CharPtr database, Boolean is_prot)
 
 	ValNodeAddPointer(&request, BlastRequest_db_info_specific, dbinfo_get);
 	SubmitRequest(bl3hp, request, &response, NULL, TRUE);
-	response = GetResponsePtr(response, BlastResponse_db_info_specific);
+	response = (BlastResponsePtr)
+           GetResponsePtr(response, BlastResponse_db_info_specific);
 
 	if (response)
 	{
@@ -935,7 +990,8 @@ BlastGetDbInfo (BlastNet3BlockPtr blnet3blkptr)
 	response = blnet3blkptr->response;
 	while (response)
 	{
-		response = GetResponsePtr(response, BlastResponse_db_info_specific);
+		response = (BlastResponsePtr)
+                   GetResponsePtr(response, BlastResponse_db_info_specific);
 		if (response)
 		{
 			last = dbinfo;
@@ -959,7 +1015,8 @@ NetBlastGetMatrix(BlastNet3BlockPtr blnet3blkptr)
 	BlastMatrixPtr matrix=NULL;
 	
 
-	response = GetResponsePtr(blnet3blkptr->response, BlastResponse_matrix);
+	response = (BlastResponsePtr)
+           GetResponsePtr(blnet3blkptr->response, BlastResponse_matrix);
 
 	if (response)
 		matrix = (BlastMatrixPtr) response->data.ptrvalue;
@@ -974,7 +1031,8 @@ BlastGetParameterBuffer (BlastNet3BlockPtr blnet3blkptr)
 	BlastResponsePtr response;
 	CharPtr buffer=NULL;
 
-	response = GetResponsePtr(blnet3blkptr->response, BlastResponse_parameters);
+	response = (BlastResponsePtr)
+           GetResponsePtr(blnet3blkptr->response, BlastResponse_parameters);
 
 	if (response)
 		buffer = (CharPtr) response->data.ptrvalue;
@@ -996,7 +1054,8 @@ BlastGetKaParams (BlastNet3BlockPtr blnet3blkptr, Boolean gapped)
 	response = blnet3blkptr->response;
 	while (response)
 	{
-		response = GetResponsePtr(response, BlastResponse_kablk);
+		response =(BlastResponsePtr)
+                   GetResponsePtr(response, BlastResponse_kablk);
 		if (response)
 		{
                     kablk = (BlastKABlkPtr) response->data.ptrvalue;
@@ -1013,6 +1072,7 @@ BlastGetKaParams (BlastNet3BlockPtr blnet3blkptr, Boolean gapped)
 /*
 	Converts 'standard' BLAST matrix to network matrix. 
 */
+
 NLM_EXTERN BlastMatrixPtr LIBCALL
 BlastMatrixToBlastNetMatrix(BLAST_MatrixPtr matrix)
 
@@ -1092,15 +1152,25 @@ BlastNetMatrixToBlastMatrix (BlastMatrixPtr net_matrix)
         posFreqs = (Nlm_FloatHi **) 
             MemNew(blast_matrix->rows*sizeof(Nlm_FloatHi *));
 
-        for (index1=0; index1 < blast_matrix->rows; index1++) {
+        for (index1=0; index1<blast_matrix->rows && vnp; index1++) {
             posFreqs[index1] = (Nlm_FloatHi *) MemNew(blast_matrix->columns*sizeof(Nlm_FloatHi));
-            for (index2=0; index2<blast_matrix->columns; index2++) {
+            for (index2=0; index2<blast_matrix->columns && vnp; index2++) {
                 posFreqs[index1][index2] = (Nlm_FloatHi) vnp->data.realvalue;
                 vnp = vnp->next;
             }
         }
         blast_matrix->posFreqs = posFreqs;
+
+	/* Check if matrix was truncated. */
+	if (index1 != blast_matrix->rows || index2 != blast_matrix->columns)
+	{
+		posFreqs = MemFree(posFreqs);
+		blast_matrix->name = MemFree(blast_matrix->name);
+		blast_matrix = MemFree(blast_matrix);
+		return NULL;
+	}
     } 
+
     if (net_matrix->scores != NULL) {
        vnp = net_matrix->scores;
        matrix = (Int4Ptr PNTR) MemNew(blast_matrix->rows*sizeof(Int4Ptr));
@@ -1130,7 +1200,8 @@ BlastGetMaskedLoc (BlastNet3BlockPtr blnet3blkptr)
 
 	while (response)
 	{
-		response = GetResponsePtr(response, BlastResponse_mask);
+		response = (BlastResponsePtr)
+                   GetResponsePtr(response, BlastResponse_mask);
 		if (response)
 		{
 			blast_mask = (BlastMaskPtr) response->data.ptrvalue;
@@ -1161,7 +1232,8 @@ PrivateBlastGetBioseq(BlastNet3Hptr bl3hptr, CharPtr database, SeqIdPtr sip, Boo
 	ValNodeAddPointer(&request, BlastRequest_db_seq_get, blast_sip);
 	SubmitRequest(bl3hptr, request, &response, NULL, TRUE);
 	
-	response = GetResponsePtr(response, BlastResponse_db_seq_get);
+	response = (BlastResponsePtr)
+           GetResponsePtr(response, BlastResponse_db_seq_get);
 	if (response)
 	{
 		bsp = (BioseqPtr) response->data.ptrvalue;
@@ -1218,36 +1290,56 @@ BlastBioseq (BlastNet3BlockPtr blnet3blkptr, ValNodePtr *error_returns, Boolean
 	search->database = blnet3blkptr->dbname;
 	search->parameters = blnet3blkptr->parameters;
 	search->mask = blnet3blkptr->mask;
-	search->matrix =
-	   BlastMatrixToBlastNetMatrix(blnet3blkptr->blast_matrix);
+	if (blnet3blkptr->blast_matrix)
+	{
+		search->matrix =
+		   BlastMatrixToBlastNetMatrix(blnet3blkptr->blast_matrix);
+	}
+	else
+	{
+		search->matrix = blnet3blkptr->net_matrix;
+	}
 	search->query_set = blnet3blkptr->bsp_set;
-	ValNodeAddPointer(&request, BlastRequest_search, search);
-	*status = SubmitRequest(blnet3blkptr->bl3hptr, request, &response, blnet3blkptr->callback, TRUE);
-	
-	blnet3blkptr->response = response;
-	node = GetResponsePtr(response, BlastResponse_alignment);
-	if (node)
-            seqalign = (SeqAlignPtr) node->data.ptrvalue;
+        
+        ValNodeAddPointer(&request, BlastRequest_search, search);
+        *status = SubmitRequest(blnet3blkptr->bl3hptr, request, &response, blnet3blkptr->callback, TRUE);
+        
+        blnet3blkptr->response = response;
+        node = (BlastResponsePtr)
+           GetResponsePtr(response, BlastResponse_alignment);
+        if (node)
+           seqalign = (SeqAlignPtr) node->data.ptrvalue;
 
         if (other_returns) { 
            /* MegaBLAST endpoint returns */
-           node = GetResponsePtr(response, BlastResponse_mbalign);
-           if (node)
-              ValNodeAddPointer(other_returns, BlastResponse_mbalign, node->data.ptrvalue);
+           node = (BlastResponsePtr)
+              GetResponsePtr(response, BlastResponse_mbalign);
+           if (node) {
+              ValNodeAddPointer(other_returns, BlastResponse_mbalign,
+                                node->data.ptrvalue);
+           }
         }
 	if (error_returns)
 	{
-	    node = GetResponsePtr(response, BlastResponse_error);
-	    if (node)
-	    	ValNodeAddPointer(error_returns, BlastResponse_error, node->data.ptrvalue);
+	  /* return all blast error responses */
+	    node = (BlastResponsePtr) 
+	      GetResponsePtr(response, BlastResponse_error);
+	    while (node)
+	    {
+		ValNodeAddPointer(error_returns, BlastResponse_error, node->data.ptrvalue);
+		node = node->next;
+	    	node = (BlastResponsePtr) 
+	      		GetResponsePtr(node, BlastResponse_error);
+	    }
 	}
 
-	/* These five are not allocated here. */
+	/* These six are not allocated here. */
 	search->query = NULL;
 	search->database = NULL;
 	search->parameters = NULL;
 	search->mask = NULL;
 	search->query_set = NULL;
+	search->matrix = NULL;
         BlastRequestFree(request);
 #if 0        
 	BlastDeleteUserErrorString(err_id);
@@ -1291,13 +1383,15 @@ BlastBioseqByParts (BlastNet3BlockPtr blnet3blkptr, ValNodePtr *error_returns, B
 	*status = SubmitRequest(blnet3blkptr->bl3hptr, request, &response, blnet3blkptr->callback, TRUE);
 	
 	blnet3blkptr->response = response;
-	node = GetResponsePtr(response, BlastResponse_parts);
+	node = (BlastResponsePtr)
+           GetResponsePtr(response, BlastResponse_parts);
 	if (node)
             blast_parts = (BlastPartsPtr) node->data.ptrvalue;
 	
 	if (error_returns)
 	{
-	    node = GetResponsePtr(response, BlastResponse_error);
+	    node = (BlastResponsePtr)
+               GetResponsePtr(response, BlastResponse_error);
 	    if (node)
 	    	ValNodeAddPointer(error_returns, BlastResponse_error, node->data.ptrvalue);
 	}
@@ -1342,12 +1436,14 @@ SeedBioseq (BlastNet3BlockPtr blnet3blkptr, ValNodePtr *error_returns,
                             blnet3blkptr->callback, TRUE);
     
     blnet3blkptr->response = response;
-    node = GetResponsePtr(response, BlastResponse_phialign);
+    node = (BlastResponsePtr)
+       GetResponsePtr(response, BlastResponse_phialign);
     if (node)
         bphp = (BlastPhialignPtr) node->data.ptrvalue;
     
     if (error_returns) {
-        node = GetResponsePtr(response, BlastResponse_error);
+        node = (BlastResponsePtr)
+           GetResponsePtr(response, BlastResponse_error);
         if (node)
             ValNodeAddPointer(error_returns, BlastResponse_error, 
                               node->data.ptrvalue);
@@ -1464,7 +1560,7 @@ SeedBioseqNetCore(BlastNet3Hptr bl3hp, BioseqPtr bsp, CharPtr program,
         if (matrix)
             ValNodeAddPointer (other_returns, TXMATRIX, matrix);
     }
-    
+
     blnet = BlastNet3BlockDestruct(blnet);
     
     bsp->descr = descr;
@@ -2413,15 +2509,12 @@ TraditionalBlastReportEngine(SeqLocPtr slp, BioseqPtr bsp, BLAST_OptionsBlkPtr o
     ReadDBBioseqFetchEnable ("blastcl3", database, db_is_na, TRUE);
 
     if (mb_results) {
+       MegaBlastHitPtr mb_hit;
        /* Results come as alignment endpoints only from Mega BLAST */
-       MegaBlastHitPtr mb_hit = mb_results->mbhits /*, next_hit */ ;
-
-
-       while (mb_hit) {
+       for (mb_hit=mb_results->mbhits; mb_hit; mb_hit = mb_hit->next) {
           fprintf(outfp, "%s\t%s\t%d\t%d\t%d\t%d\t%d\n", mb_hit->id1,
                   mb_hit->id2, mb_hit->query_offset, mb_hit->subject_offset,
                   mb_hit->query_end, mb_hit->subject_end, mb_hit->score);
-          mb_hit = mb_hit->next;
        }
        MegaBlastResultsFree(mb_results);
     }
@@ -2571,7 +2664,7 @@ TraditionalBlastReportLocExtra(SeqLocPtr slp, BLAST_OptionsBlkPtr options, Blast
         Converst the BlastParametersPtr (used by network service) to
         BLAST_OptionsBlkPtr (used by blast).
 */
-
+#ifndef SPLIT_BLAST
 NLM_EXTERN BLAST_OptionsBlkPtr
 parametersToOptions (BlastParametersPtr parameters, CharPtr program, ValNodePtr PNTR error_return)
 
@@ -2646,7 +2739,9 @@ parametersToOptions (BlastParametersPtr parameters, CharPtr program, ValNodePtr
                     options->searchsp_eff = parameters->searchsp_eff;
 		if (parameters->hsp_range_max)
                     options->hsp_range_max = parameters->hsp_range_max;
-		if (parameters->block_width)
+		if (parameters->block_width || parameters->is_megablast)
+                   /* In case of megablast, block_width should be 0 by default
+                      instead of 20 returned from BLASTOptionNew */
                     options->block_width = parameters->block_width;
 		if (parameters->perform_culling)
                     options->perform_culling = parameters->perform_culling;
@@ -2676,8 +2771,20 @@ parametersToOptions (BlastParametersPtr parameters, CharPtr program, ValNodePtr
 		options->is_megablast_search = parameters->is_megablast;
                 options->query_lcase_mask = 
                    (SeqLocPtr) parameters->query_lcase_mask;
-                options->no_traceback = parameters->endpoint_results;
+                if (parameters->endpoint_results)
+                   options->no_traceback = 1; 
                 options->perc_identity = (FloatLo) parameters->percent_identity;
+                options->first_db_seq = parameters->first_db_seq;
+                options->final_db_seq = parameters->final_db_seq;
+                options->window_size = parameters->window_size;
+                if (options->window_size == 0 && StringCmp(program, "blastn"))
+                    options->window_size = 40;
+                options->mb_template_length = parameters->mb_template_length;
+                options->mb_disc_type = parameters->mb_disc_type;
+                if (options->mb_template_length > 0 && 
+                    (options->gap_open > 0 || options->gap_extend > 0)) {
+                   options->mb_use_dyn_prog = TRUE;
+                }
         }
 
 	if ((status = BLASTOptionValidateEx(options, program, error_return))) {
@@ -2690,8 +2797,11 @@ parametersToOptions (BlastParametersPtr parameters, CharPtr program, ValNodePtr
    to get list of gis corresponding to the Entrez Boolean string or
    just number of such hits in the Entrez database */
 
-NLM_EXTERN Int4 BLASTGetUidsFromQuery(CharPtr query, Int4Ptr PNTR uids, 
-                                      Boolean is_na, Boolean count_only)
+NLM_EXTERN Int4 BLASTGetUidsFromQuery(CharPtr       query,
+				      Int4Ptr PNTR  uids,
+                                      Boolean       is_na,
+				      Boolean       count_only,
+				      BlastError ** blast_err)
 {
     Entrez2ReplyPtr e2ry;
     Entrez2RequestPtr  e2rq;
@@ -2714,20 +2824,55 @@ NLM_EXTERN Int4 BLASTGetUidsFromQuery(CharPtr query, Int4Ptr PNTR uids,
     e2ry = EntrezSynchronousQuery (e2rq);
     
     if (e2ry == NULL) {
-        ErrPostEx(SEV_ERROR, 0, 0, 
-                  "NULL returned from EntrezSynchronousQuery()");
+	if ( blast_err ) {
+	    *blast_err = BlastErrorNew();
+	    
+	    (*blast_err)->level = Blast_error_level_error;
+	    (*blast_err)->msg   = StrSave("NULL returned from EntrezSynchronousQuery()");
+	} else {
+	    ErrPostEx(SEV_ERROR, 0, 0, 
+		      "NULL returned from EntrezSynchronousQuery()");
+	}
+	
         return -1;
     }
-
+    
     if((e2rp = e2ry->reply) == NULL) {
-        ErrPostEx(SEV_ERROR, 0, 0, "Invalid ASN.1: E2ReplyPtr==NULL");
+	if ( blast_err ) {
+	    *blast_err = BlastErrorNew();
+	    
+	    (*blast_err)->level = Blast_error_level_error;
+	    (*blast_err)->msg   = StrSave("Invalid ASN.1: E2ReplyPtr==NULL");
+	} else {
+	    ErrPostEx(SEV_ERROR, 0, 0, "Invalid ASN.1: E2ReplyPtr==NULL");
+	}
         return -1;
     }
     
     switch(e2rp->choice) {
-        
     case E2Reply_error:
-        ErrPostEx(SEV_ERROR, 0, 0, (CharPtr) e2rp->data.ptrvalue);
+	if ( blast_err ) {
+	    const char * msg1 = "Error in request: ";
+	    const char * msg2 = (CharPtr) e2rp->data.ptrvalue;
+	    char       * msg3 = 0;
+	    
+	    int msglen1 = strlen(msg1);
+	    int msglen2 = strlen(msg2);
+	    
+	    msg3 = MemNew(msglen1 + msglen2 + 1);
+	    
+	    if ( msg3 ) {
+		strcpy(msg3, msg1);	
+		strcat(msg3, msg2);
+		
+		*blast_err = BlastErrorNew();
+		
+		(*blast_err)->level = Blast_error_level_error;
+		(*blast_err)->msg   = msg3;
+	    }
+	} else {
+	    ErrPostEx(SEV_ERROR, 0, 0, "Error in request: %s", (CharPtr) e2rp->data.ptrvalue);
+	}
         count = -1;
         break;
     case E2Reply_eval_boolean:
@@ -2743,14 +2888,36 @@ NLM_EXTERN Int4 BLASTGetUidsFromQuery(CharPtr query, Int4Ptr PNTR uids,
         }
         break;
     default:
-        ErrPostEx(SEV_ERROR, 0, 0, "Invalid reply type from the server: %d", e2rp->choice);
+	if ( blast_err ) {
+	    const char * msg1 = "Invalid reply type from the server: ";
+	    const char * msg2 = (CharPtr) e2rp->data.ptrvalue;
+	    char       * msg3 = 0;
+	    
+	    int msglen1 = strlen(msg1);
+	    int msglen2 = strlen(msg2);
+	    
+	    msg3 = MemNew(msglen1 + msglen2 + 1);
+	    
+	    if ( msg3 ) {
+		strcpy(msg3, msg1);	
+		strcat(msg3, msg2);
+		
+		*blast_err = BlastErrorNew();
+		
+		(*blast_err)->level = Blast_error_level_error;
+		(*blast_err)->msg   = msg3;
+	    }
+	} else {
+	    ErrPostEx(SEV_ERROR, 0, 0, "Invalid reply type from the server: %d", e2rp->choice);
+	}
         count = -1;
         break;
-        
     }
     
     Entrez2ReplyFree(e2ry);
     Entrez2RequestFree(e2rq);
-
+    
     return count;
 }
+#endif
+
