@@ -29,7 +29,7 @@
 *   
 * Version Creation Date: 4/1/91
 *
-* $Revision: 6.6 $
+* $Revision: 6.8 $
 *
 * File Description:  Object manager interface for type Pubdesc from
 *                    NCBI-Sequence.  This is separate to avoid typedef
@@ -45,6 +45,12 @@
 * -------  ----------  -----------------------------------------------------
 *
 * $Log: objpubd.h,v $
+* Revision 6.8  2004/12/08 20:20:34  kans
+* changed MI_TECH_composite to MI_TECH_composite_wgs_htgs
+*
+* Revision 6.7  2004/12/08 19:59:24  kans
+* added MI_TECH_composite
+*
 * Revision 6.6  2004/08/30 18:08:52  kans
 * added MI_TECH_barcode
 *
@@ -237,6 +243,7 @@ typedef struct molinfo {
 #define MI_TECH_htc 19       /* High Throughput CDNA.. unfinished */
 #define MI_TECH_wgs 20       /* Whole Genome Shotgun */
 #define MI_TECH_barcode 21   /* Barcode of Life Project */
+#define MI_TECH_composite_wgs_htgs 22   /* Assembly/Scaffold composite of techniques */
 #define MI_TECH_other 255           /* doesnt' fit anything */
 
 NLM_EXTERN MolInfoPtr LIBCALL MolInfoNew PROTO((void));
