@@ -1,6 +1,6 @@
-static char const rcsid[] = "$Id: xmlblast.c,v 6.35 2004/06/30 12:32:20 madden Exp $";
+static char const rcsid[] = "$Id: xmlblast.c,v 6.36 2004/10/20 19:58:58 dondosha Exp $";
 
-/* $Id: xmlblast.c,v 6.35 2004/06/30 12:32:20 madden Exp $ */
+/* $Id: xmlblast.c,v 6.36 2004/10/20 19:58:58 dondosha Exp $ */
 /**************************************************************************
 *                                                                         *
 *                             COPYRIGHT NOTICE                            *
@@ -32,12 +32,15 @@ static char const rcsid[] = "$Id: xmlblast.c,v 6.35 2004/06/30 12:32:20 madden E
 *   
 * Version Creation Date: 05/17/2000
 *
-* $Revision: 6.35 $
+* $Revision: 6.36 $
 *
 * File Description:  Functions to print simplified BLAST output (XML)
 *
 * 
 * $Log: xmlblast.c,v $
+* Revision 6.36  2004/10/20 19:58:58  dondosha
+* Replace sequence data in Bioseq, as done in txalign.c, to assure proper masking and identities/positives calculation for all programs
+*
 * Revision 6.35  2004/06/30 12:32:20  madden
 * Added include for blfmtutl.h, removed unused variable
 *
@@ -592,50 +595,21 @@ Boolean BXMLGetSeqLines(SeqAlignPtr align, HspPtr hsp, Int4 length,
     return TRUE;
 }
 
-static void 
-MaskFilteredLocInHsp(HspPtr hsp, ValNodePtr mask, Boolean is_prot,
-                     Boolean query_translated)
+/** Frees the part of the byte store list that has not been used for replacement
+ * of Bioseq data.
+ */
+static ValNodePtr 
+ByteStoreListFree(ValNodePtr bs_list)
 {
-   Char mask_char;
-   SeqLocPtr mask_loc;
-   SeqLocPtr seqloc = NULL;
-   Int4 start, stop, index;
+   ByteStorePtr bsp;
+   ValNodePtr vnp;
 
-   if (!mask)
-      return;
-
-   mask_char = (is_prot ? 'X' : 'N');
-
-   for ( ; mask; mask = mask->next) {
-      if (mask->choice != FrameToDefine(hsp->query_frame))
-         continue;
-      mask_loc = (SeqLocPtr) mask->data.ptrvalue;
-
-      while((seqloc = SeqLocFindNext(mask_loc, seqloc)) != NULL) {
-         if (hsp->query_from < hsp->query_to) {
-            start = MAX(hsp->query_from, SeqLocStart(seqloc))
-               - hsp->query_from;
-            stop = MIN(SeqLocStop(seqloc), hsp->query_to) - 
-               hsp->query_from;
-         } else {
-            start = hsp->query_from - 
-               MIN(SeqLocStop(seqloc), hsp->query_from);
-            stop = hsp->query_from - 
-               MAX(hsp->query_to, SeqLocStart(seqloc));
-         }
-
-         if (query_translated) {
-            start /= CODON_LENGTH;
-            stop /= CODON_LENGTH;
-         }
-         
-         for (index = start; index <= stop; ++index) {
-            hsp->qseq[index] = mask_char;
-            if (hsp->hseq[index] != mask_char)
-               hsp->midline[index] = ' ';
-         }
-      }
+   for(vnp = bs_list; vnp; vnp = vnp->next) {
+      bsp = (ByteStorePtr) vnp->data.ptrvalue;
+      if(bsp != NULL)
+          BSFree(bsp);
    }
+   return ValNodeFree(bs_list);
 }
 
 HspPtr BXMLGetHspFromSeqAlign(SeqAlignPtr sap, Boolean is_aa, Int4 chain,
@@ -645,10 +619,13 @@ HspPtr BXMLGetHspFromSeqAlign(SeqAlignPtr sap, Boolean is_aa, Int4 chain,
     AlignSum as;
     ScorePtr score, sp;
     Boolean matrix_allocated = FALSE;
-    BioseqPtr bsp;
+    BioseqPtr bsp = NULL;
     Char tmp[256];
-    static Boolean master_checked;
-    Boolean prot_alphabet;
+    ByteStorePtr seq_data=NULL;
+    Boolean seq_data_replaced = FALSE;
+    ValNodePtr bs_list = NULL;
+    Uint1 code=0;
+    Uint1 repr=0;
 
     if((hsp = HspNew()) == NULL)
         return NULL;
@@ -658,19 +635,49 @@ HspPtr BXMLGetHspFromSeqAlign(SeqAlignPtr sap, Boolean is_aa, Int4 chain,
     as.master_sip = TxGetQueryIdFromSeqAlign(sap);
     as.target_sip = TxGetSubjectIdFromSeqAlign(sap);
 
-    /* Checkup for query Bioseq to be available in the index */
+    /* If there is a mask, retrieve the query Bioseq and replace the sequence
+       data with a masked one. */
+    if (mask_loc) {
+        Int4 frame;
+        if ((bsp = BioseqLockById(as.master_sip)) == NULL) {
+            SeqIdWrite(as.master_sip, tmp, PRINTID_FASTA_LONG, sizeof(tmp));
+            ErrPostEx(SEV_ERROR, 0, __LINE__, "Query sequence '%s' is not "
+                      "available in Bioseq index.", tmp);
+            return NULL;
+        }
 
-    if(!master_checked && (bsp = BioseqLockById(as.master_sip)) == NULL) {
-        SeqIdWrite(as.master_sip, tmp, PRINTID_FASTA_LONG, sizeof(tmp));
-        ErrPostEx(SEV_ERROR, 0, __LINE__, "Query sequence '%s' is not "
-                  "available in Bioseq index.", tmp);
-        return NULL;
-    } else if(!master_checked) {
-        master_checked = TRUE;
-        BioseqUnlock(bsp);
+        bs_list = CreateMaskByteStore (mask_loc);
+        if (ISA_na(bsp->mol) && sap->segtype == SAS_STD) {
+            StdSegPtr ssp = (StdSegPtr) sap->segs;
+            frame = SeqLocStart(ssp->loc);
+            if (SeqLocStrand(ssp->loc) == Seq_strand_minus) {
+                frame += SeqLocLen(ssp->loc);
+                frame = 4 + (bsp->length - frame)%3;
+            } else {
+                frame = (1 + frame%3);
+            }
+        } else
+            frame = 0;
+        repr = bsp->repr;
+        seq_data = bsp->seq_data;
+        code = bsp->seq_data_type;
+        seq_data_replaced = 
+                replace_bytestore_data(bsp, bs_list, (Uint1)frame);
+        if (!seq_data_replaced) {
+            bsp->repr = repr;
+            bsp->seq_data = seq_data;
+            bsp->seq_data_type = code;
+        }
+    } else {
+        /* Checkup for query Bioseq to be available in the index */
+        if ((bsp = BioseqLockById(as.master_sip)) == NULL) {
+            SeqIdWrite(as.master_sip, tmp, PRINTID_FASTA_LONG, sizeof(tmp));
+            ErrPostEx(SEV_ERROR, 0, __LINE__, "Query sequence '%s' is not "
+                      "available in Bioseq index.", tmp);
+            return NULL;
+        }
     }
 
-
     if(glb_matrix != NULL)
         as.matrix = glb_matrix;
     else {
@@ -727,11 +734,16 @@ HspPtr BXMLGetHspFromSeqAlign(SeqAlignPtr sap, Boolean is_aa, Int4 chain,
     hsp->hit_to = as.target_to;
 
     BXMLGetSeqLines(sap, hsp, as.totlen, is_aa, chain, as.matrix);
-    /* Sequences are shown in protein alphabet in all cases except blastn,
-       that is if either subject is protein, or it is a translated search, 
-       in which case alignment segments have StdSeg form. */
-    prot_alphabet = (is_aa || sap->segtype == SAS_STD);
-    MaskFilteredLocInHsp(hsp, mask_loc, prot_alphabet, as.m_frame_set);
+
+    /* Restore the original query sequence data, if it was replaced. */
+    if (seq_data_replaced) {
+        bsp->seq_data = seq_data;
+        bsp->repr = repr;
+        bsp->seq_data_type = code;
+    }
+    bs_list = ByteStoreListFree(bs_list);
+    BioseqUnlock(bsp);
+
 
     /* For display it depends on strand */
     
@@ -899,6 +911,7 @@ IterationPtr BXMLBuildOneQueryIteration(SeqAlignPtr seqalign,
        }
 
        iterp->query_len = query->length;
+
     }
 
     if(seqalign != NULL) {
